"""
Agent 核心类 - 对应 packages/core/src/agent/agent.ts

这是 PyMidscene 的核心入口，提供 AI 驱动的自动化能力。
"""

from typing import Optional, Dict, Any, List, Union, Tuple
import time
import asyncio

from ..ai_model import call_ai
from ..ai_model.prompts import (
    system_prompt_to_locate_element,
    find_element_prompt,
    system_prompt_to_extract,
    extract_data_prompt,
    parse_xml_extraction_response,
)
from ..agent.task_cache import TaskCache, PlanningCache, LocateCache
from ..dump import ExecutionRecorder, SessionRecorder, create_session_recorder
from ..types import ScreenshotItem
from ...web_integration.base import AbstractInterface
from ...shared.types import Size, Rect, LocateResultElement
from ...shared.logger import logger
from ...shared.utils import calculate_center, format_bbox, adapt_bbox
from ...shared.env import (
    ModelConfigManager,
    ModelConfig,
    get_global_model_config_manager,
    INTENT_DEFAULT,
)


class Agent:
    """
    AI 驱动的自动化 Agent

    整合了 AI 模型、缓存系统和浏览器控制，提供高级的自动化能力。

    使用方式：
    1. 通过环境变量配置模型（推荐）：
        os.environ["MIDSCENE_MODEL_NAME"] = "qwen-vl-max"
        os.environ["OPENAI_API_KEY"] = "your-key"
        os.environ["OPENAI_BASE_URL"] = "https://dashscope.aliyuncs.com/compatible-mode/v1"
        os.environ["MIDSCENE_MODEL_FAMILY"] = "qwen2.5-vl"

        agent = Agent(interface)

    2. 通过 model_config 字典配置：
        agent = Agent(interface, model_config={
            "MIDSCENE_MODEL_NAME": "qwen-vl-max",
            "OPENAI_API_KEY": "your-key",
            "OPENAI_BASE_URL": "https://...",
            "MIDSCENE_MODEL_FAMILY": "qwen2.5-vl",
        })
    """

    def __init__(
        self,
        interface: AbstractInterface,
        model_config: Optional[Dict[str, Any]] = None,
        cache_id: Optional[str] = None,
        cache_strategy: str = "read-write",
        cache_dir: Optional[str] = None,
        enable_recording: bool = True,  # 默认启用记录
        driver_type: str = "playwright",
        report_dir: Optional[str] = None,
    ):
        """
        初始化 Agent

        Args:
            interface: 设备接口（WebPage 等）
            model_config: 模型配置字典，键为环境变量名称
                - MIDSCENE_MODEL_NAME: 模型名称
                - OPENAI_API_KEY / MIDSCENE_MODEL_API_KEY: API 密钥
                - OPENAI_BASE_URL / MIDSCENE_MODEL_BASE_URL: API 基础 URL
                - MIDSCENE_MODEL_FAMILY: 模型家族（qwen2.5-vl, doubao-vision 等）
            cache_id: 缓存 ID
            cache_strategy: 缓存策略（read-only, read-write, write-only）
            cache_dir: 缓存目录
            enable_recording: 是否启用执行记录（默认启用）
            driver_type: 驱动类型（playwright, selenium 等）
            report_dir: 报告保存目录（默认为当前目录）
        """
        self.interface = interface
        self.driver_type = driver_type

        # 初始化模型配置管理器
        if model_config:
            # 使用传入的配置（隔离模式）
            self.model_config_manager = ModelConfigManager(model_config)
        else:
            # 使用全局配置管理器（从环境变量读取）
            self.model_config_manager = get_global_model_config_manager()

        # 初始化缓存
        self.task_cache: Optional[TaskCache] = None
        if cache_id:
            self.task_cache = TaskCache(
                cache_id=cache_id,
                is_cache_result_used=(cache_strategy != "write-only"),
                cache_dir=cache_dir,
                strategy=cache_strategy
            )
            logger.info(f"Task cache initialized: {cache_id}")

        # 初始化会话记录器（新的日志系统）
        self.enable_recording = enable_recording
        self.session_recorder: Optional[SessionRecorder] = None
        self.recorder: Optional[ExecutionRecorder] = None  # 保持向后兼容

        if enable_recording:
            self.session_recorder = create_session_recorder(
                driver_type=driver_type,
                base_dir=report_dir,
                auto_save=True
            )
            # 同时保留旧的 recorder 以兼容
            self.recorder = ExecutionRecorder(
                name="Agent Execution",
                description="Automated execution"
            )
            logger.info(f"Session recording enabled: {self.session_recorder.session_id}")

        logger.info(
            f"Agent initialized: "
            f"cache={'enabled' if cache_id else 'disabled'}, "
            f"recording={'enabled' if enable_recording else 'disabled'}"
        )

    def _get_model_config(self, intent: str = INTENT_DEFAULT) -> ModelConfig:
        """获取模型配置"""
        return self.model_config_manager.get_model_config(intent)

    def _build_messages(
        self,
        system_prompt: str,
        user_prompt: str,
        screenshot_b64: str
    ) -> List[Dict[str, Any]]:
        """构建 AI 消息"""
        return [
            {
                "role": "system",
                "content": system_prompt
            },
            {
                "role": "user",
                "content": [
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": f"data:image/png;base64,{screenshot_b64}"
                        }
                    },
                    {
                        "type": "text",
                        "text": user_prompt
                    }
                ]
            }
        ]

    def _call_ai_with_config(
        self,
        messages: List[Dict[str, Any]],
        intent: str = INTENT_DEFAULT
    ) -> Dict[str, Any]:
        """使用配置调用 AI"""
        config = self._get_model_config(intent)

        # 构建调用参数
        from openai import OpenAI

        client = OpenAI(
            api_key=config.openai_api_key,
            base_url=config.openai_base_url,
        )

        response = client.chat.completions.create(
            model=config.model_name,
            messages=messages,
            temperature=config.temperature,
        )

        content = response.choices[0].message.content or ""

        # 提取 usage 信息
        usage = None
        if hasattr(response, 'usage') and response.usage:
            usage = {
                "prompt_tokens": response.usage.prompt_tokens,
                "completion_tokens": response.usage.completion_tokens,
                "total_tokens": response.usage.total_tokens,
            }

        return {
            "content": content,
            "usage": usage,
            "raw_response": response
        }

    async def ai_locate(
        self,
        prompt: str,
        use_cache: bool = True
    ) -> Optional[LocateResultElement]:
        """
        使用 AI 定位页面元素

        Args:
            prompt: 元素描述（如 "登录按钮"）
            use_cache: 是否使用缓存

        Returns:
            定位结果或 None
        """
        logger.info(f"AI Locate: '{prompt}'")

        # 开始记录步骤（SessionRecorder）
        if self.session_recorder:
            self.session_recorder.start_step("locate", prompt)

        # 开始记录任务（旧版兼容）
        if self.recorder:
            task = self.recorder.start_task("locate", param=prompt)

        # 检查缓存（与 JS 版本对齐：使用 XPath 定位）
        if use_cache and self.task_cache:
            cache_result = self.task_cache.match_locate_cache(prompt)
            if cache_result:
                logger.info(f"Cache hit for locate: '{prompt}'")
                cache_data = cache_result.cache_content.cache
                
                # JS 版本格式：使用 xpaths 定位元素
                if cache_data and "xpaths" in cache_data:
                    xpaths = cache_data["xpaths"]
                    if xpaths and len(xpaths) > 0:
                        xpath = xpaths[0]  # 使用第一个 XPath
                        logger.info(f"Using cached XPath: {xpath[:80]}...")
                        
                        # 通过 XPath 获取元素位置
                        element_info = await self.interface.get_element_by_xpath(xpath)
                        if element_info:
                            rect = element_info["rect"]
                            center = element_info["center"]
                            
                            element = LocateResultElement(
                                description=prompt,
                                center=center,
                                rect=rect
                            )
                            
                            logger.info(f"Cache hit (XPath): '{prompt}' at {center}")
                            
                            # 记录缓存命中
                            if self.recorder:
                                self.recorder.finish_task(status="finished", output=element)
                            if self.session_recorder:
                                self.session_recorder.complete_step("success")
                            
                            return element
                        else:
                            # XPath 找不到元素，可能页面结构变化，需要重新 AI 定位
                            logger.warning(f"Cached XPath not found on page, re-locating: '{prompt}'")

        # 获取截图
        screenshot_b64 = await self.interface.screenshot()
        size = await self.interface.get_size()

        # 记录截图（SessionRecorder）
        if self.session_recorder:
            self.session_recorder.record_screenshot_before(screenshot_b64)

        # 记录截图（旧版兼容）
        if self.recorder:
            screenshot_item = ScreenshotItem(screenshot_b64)
            self.recorder.record_screenshot(screenshot_item, timing="before")

        # 获取模型配置
        config = self._get_model_config(INTENT_DEFAULT)
        model_family = config.model_family or "qwen2.5-vl"

        # 准备消息
        messages = self._build_messages(
            system_prompt=system_prompt_to_locate_element(model_family),
            user_prompt=find_element_prompt(prompt),
            screenshot_b64=screenshot_b64
        )

        # 调用 AI
        start_time = time.time()
        result = self._call_ai_with_config(messages, INTENT_DEFAULT)
        elapsed_ms = (time.time() - start_time) * 1000

        # 记录 AI 使用信息
        if self.session_recorder and result.get('usage'):
            self.session_recorder.record_ai_info(
                model=config.model_name,
                tokens=result['usage'].get('total_tokens'),
                response=result.get('content', '')[:500]
            )

        if self.recorder and result.get('usage'):
            usage_info = result['usage'].copy()
            usage_info['time_cost'] = elapsed_ms / 1000
            usage_info['model_name'] = config.model_name
            self.recorder.record_ai_usage(usage_info)

        logger.info(
            f"AI locate completed: {elapsed_ms:.0f}ms, "
            f"tokens={result.get('usage', {}).get('total_tokens', 0) if result.get('usage') else 0}"
        )

        # 解析结果
        from ...shared.utils import safe_parse_json, extract_json_from_code_block
        # 先提取 JSON（处理 markdown 代码块）
        json_text = extract_json_from_code_block(result["content"])
        response_data = safe_parse_json(json_text)

        if not response_data or "bbox" not in response_data:
            logger.warning(f"AI locate failed: no bbox in response")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=ValueError("No bbox in response"))
            if self.session_recorder:
                self.session_recorder.fail_step("No bbox in response")
            return None

        bbox = response_data["bbox"]
        if not bbox or (isinstance(bbox, list) and len(bbox) < 2):
            logger.warning(f"AI locate failed: invalid bbox {bbox}")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=ValueError(f"Invalid bbox: {bbox}"))
            if self.session_recorder:
                self.session_recorder.fail_step(f"Invalid bbox: {bbox}")
            return None

        # 根据模型类型适配 bbox 坐标（关键：doubao 返回的是归一化 0-1000 坐标）
        model_family = config.model_family or "qwen2.5-vl"
        img_width = int(size.get('width', 1280))
        img_height = int(size.get('height', 800))
        
        try:
            adapted_bbox = adapt_bbox(bbox, img_width, img_height, model_family)
            logger.debug(f"Adapted bbox: {bbox} -> {adapted_bbox} (model={model_family}, size={img_width}x{img_height})")
        except Exception as e:
            logger.warning(f"Failed to adapt bbox: {e}, using raw values")
            adapted_bbox = tuple(bbox) if isinstance(bbox, list) else bbox

        # 转换为 Rect 和中心点
        rect = format_bbox(adapted_bbox)
        center = calculate_center(rect)

        element = LocateResultElement(
            description=prompt,
            center=center,
            rect=rect
        )

        # 记录元素定位结果（SessionRecorder - 带可视化标记）
        if self.session_recorder:
            bbox_tuple: Tuple[int, int, int, int] = (
                int(rect['left']),
                int(rect['top']),
                int(rect['left'] + rect['width']),
                int(rect['top'] + rect['height'])
            )
            center_tuple: Tuple[int, int] = (int(center[0]), int(center[1]))
            self.session_recorder.record_element_location(
                bbox=bbox_tuple,
                center=center_tuple,
                description=prompt,
                draw_marker=True
            )

        # 保存到缓存（与 JS 版本对齐：存储 XPath 而不是坐标）
        if use_cache and self.task_cache:
            # 获取元素的 XPath（通过元素中心点定位 DOM 元素）
            xpath = await self.interface.get_element_xpath(center[0], center[1])
            if xpath:
                self.task_cache.append_cache(LocateCache(
                    type="locate",
                    prompt=prompt,
                    cache={
                        "xpaths": [xpath]  # 与 JS 版本格式完全一致
                    }
                ))
                logger.debug(f"Cached XPath for '{prompt}': {xpath[:80]}...")
            else:
                logger.warning(f"Could not get XPath for element at {center}, cache not saved")

        # 完成任务记录
        if self.recorder:
            self.recorder.finish_task(status="finished", output=element)
        if self.session_recorder:
            self.session_recorder.complete_step("success")

        logger.info(f"Element located: {prompt} at {center}")

        return element

    async def ai_click(self, prompt: str) -> bool:
        """
        使用 AI 定位并点击元素

        Args:
            prompt: 元素描述

        Returns:
            是否成功
        """
        # 开始记录步骤（SessionRecorder）
        if self.session_recorder:
            self.session_recorder.start_step("click", prompt)
            # 获取操作前截图
            screenshot_before = await self.interface.screenshot()
            self.session_recorder.record_screenshot_before(screenshot_before)

        # 开始记录任务（旧版兼容）
        if self.recorder:
            task = self.recorder.start_task("click", param=prompt)

        element = await self.ai_locate(prompt)
        if not element:
            logger.error(f"Cannot locate element: {prompt}")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=ValueError(f"Cannot locate: {prompt}"))
            if self.session_recorder:
                self.session_recorder.fail_step(f"Cannot locate: {prompt}")
            return False

        # 记录元素位置
        if self.session_recorder:
            rect = element.rect
            bbox_tuple: Tuple[int, int, int, int] = (
                int(rect['left']),
                int(rect['top']),
                int(rect['left'] + rect['width']),
                int(rect['top'] + rect['height'])
            )
            center_tuple: Tuple[int, int] = (int(element.center[0]), int(element.center[1]))
            self.session_recorder.record_element_location(
                bbox=bbox_tuple,
                center=center_tuple,
                description=prompt,
                draw_marker=True
            )

        # 点击中心点
        x, y = element.center
        await self.interface.click(x, y)

        # 获取操作后截图
        if self.session_recorder:
            screenshot_after = await self.interface.screenshot()
            self.session_recorder.record_screenshot_after(screenshot_after)
            self.session_recorder.complete_step("success")

        logger.info(f"Clicked: {prompt} at ({x}, {y})")

        # 完成任务记录
        if self.recorder:
            self.recorder.finish_task(status="finished", output={"x": x, "y": y})

        return True

    async def ai_input(self, prompt: str, text: str) -> bool:
        """
        使用 AI 定位并输入文本

        Args:
            prompt: 元素描述
            text: 要输入的文本

        Returns:
            是否成功
        """
        # 开始记录步骤（SessionRecorder）
        if self.session_recorder:
            self.session_recorder.start_step("input", f"{prompt}: {text}")
            screenshot_before = await self.interface.screenshot()
            self.session_recorder.record_screenshot_before(screenshot_before)

        # 开始记录任务（旧版兼容）
        if self.recorder:
            task = self.recorder.start_task("input", param={"prompt": prompt, "text": text})

        element = await self.ai_locate(prompt)
        if not element:
            logger.error(f"Cannot locate element: {prompt}")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=ValueError(f"Cannot locate: {prompt}"))
            if self.session_recorder:
                self.session_recorder.fail_step(f"Cannot locate: {prompt}")
            return False

        # 点击并输入
        x, y = element.center
        await self.interface.input_text(text, x, y)

        # 获取操作后截图
        if self.session_recorder:
            screenshot_after = await self.interface.screenshot()
            self.session_recorder.record_screenshot_after(screenshot_after)
            self.session_recorder.complete_step("success")

        logger.info(f"Input to {prompt}: '{text}'")

        # 完成任务记录
        if self.recorder:
            self.recorder.finish_task(status="finished", output={"text": text})

        return True

    async def ai_query(
        self,
        data_demand: Union[Dict[str, str], str],
        use_cache: bool = False
    ) -> Dict[str, Any]:
        """
        使用 AI 从页面提取数据

        Args:
            data_demand: 数据需求（字典或字符串）
            use_cache: 是否使用缓存

        Returns:
            提取的数据
        """
        logger.info(f"AI Query: {data_demand}")

        # 开始记录任务
        if self.recorder:
            task = self.recorder.start_task("query", param=data_demand)

        # 获取截图
        screenshot_b64 = await self.interface.screenshot()

        # 记录截图
        if self.recorder:
            screenshot_item = ScreenshotItem(screenshot_b64)
            self.recorder.record_screenshot(screenshot_item, timing="before")

        # 准备消息
        messages = self._build_messages(
            system_prompt=system_prompt_to_extract(),
            user_prompt=extract_data_prompt(data_demand),
            screenshot_b64=screenshot_b64
        )

        # 调用 AI
        start_time = time.time()
        config = self._get_model_config(INTENT_DEFAULT)
        result = self._call_ai_with_config(messages, INTENT_DEFAULT)
        elapsed_ms = (time.time() - start_time) * 1000

        # 记录 AI 使用信息
        if self.recorder and result.get('usage'):
            usage_info = result['usage'].copy()
            usage_info['time_cost'] = elapsed_ms / 1000
            usage_info['model_name'] = config.model_name
            self.recorder.record_ai_usage(usage_info)

        logger.info(
            f"AI query completed: {elapsed_ms:.0f}ms, "
            f"tokens={result.get('usage', {}).get('total_tokens', 0) if result.get('usage') else 0}"
        )

        # 解析 XML 响应
        try:
            parsed = parse_xml_extraction_response(result["content"])
            logger.info(f"Data extracted: {parsed['data']}")

            # 完成任务记录
            if self.recorder:
                self.recorder.finish_task(status="finished", output=parsed)

            return parsed
        except Exception as e:
            logger.error(f"Failed to parse extraction response: {e}")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=e)
            raise

    async def ai_assert(
        self,
        assertion: str,
        message: str = ""
    ) -> bool:
        """
        使用 AI 断言页面状态

        Args:
            assertion: 断言描述（如 "页面显示登录成功"）
            message: 错误消息

        Returns:
            断言是否通过

        Raises:
            AssertionError: 如果断言失败
        """
        logger.info(f"AI Assert: '{assertion}'")

        # 开始记录任务
        if self.recorder:
            task = self.recorder.start_task("assert", param=assertion)

        # 获取截图
        screenshot_b64 = await self.interface.screenshot()

        # 记录截图
        if self.recorder:
            screenshot_item = ScreenshotItem(screenshot_b64)
            self.recorder.record_screenshot(screenshot_item, timing="before")

        # 准备消息
        messages = self._build_messages(
            system_prompt='You are an AI assistant that verifies UI states. Return JSON: {"pass": true/false, "thought": "reasoning"}',
            user_prompt=f"Verify: {assertion}",
            screenshot_b64=screenshot_b64
        )

        # 调用 AI
        start_time = time.time()
        config = self._get_model_config(INTENT_DEFAULT)
        result = self._call_ai_with_config(messages, INTENT_DEFAULT)
        elapsed_ms = (time.time() - start_time) * 1000

        # 记录 AI 使用信息
        if self.recorder and result.get('usage'):
            usage_info = result['usage'].copy()
            usage_info['time_cost'] = elapsed_ms / 1000
            usage_info['model_name'] = config.model_name
            self.recorder.record_ai_usage(usage_info)

        # 解析结果
        from ...shared.utils import safe_parse_json
        response_data = safe_parse_json(result["content"])

        if not response_data:
            error = ValueError("Failed to parse assertion response")
            if self.recorder:
                self.recorder.finish_task(status="failed", error=error)
            raise error

        passed = response_data.get("pass", False)
        thought = response_data.get("thought", "")

        if passed:
            logger.info(f"Assertion passed: {assertion}")
            if self.recorder:
                self.recorder.finish_task(status="finished", output={"pass": True, "thought": thought})
            return True
        else:
            error_msg = message or f"Assertion failed: {assertion}\nReason: {thought}"
            logger.error(error_msg)
            if self.recorder:
                self.recorder.finish_task(status="failed", error=AssertionError(error_msg))
            raise AssertionError(error_msg)

    async def ai_act(
        self,
        task_prompt: str,
        use_cache: bool = True
    ) -> bool:
        """
        使用 AI 执行复杂任务（简化版）

        Args:
            task_prompt: 任务描述
            use_cache: 是否使用缓存

        Returns:
            是否成功
        """
        logger.info(f"AI Act: '{task_prompt}'")

        # 这是简化版实现
        # 完整实现需要：
        # 1. 任务规划（生成 YAML 工作流）
        # 2. 任务执行器（执行每个步骤）
        # 3. 错误处理和重试

        # 目前仅作为示例，实际应该调用规划和执行逻辑
        logger.warning("ai_act is simplified version, full implementation pending")

        return True

    def get_cache_stats(self) -> Optional[Dict[str, Any]]:
        """获取缓存统计信息"""
        if self.task_cache:
            return self.task_cache.get_stats()
        return None

    def finish(self) -> Optional[str]:
        """
        结束会话并生成报告

        Returns:
            报告文件路径（如果启用了记录）
        """
        if self.session_recorder:
            report_path = self.session_recorder.finish()
            logger.info(f"Session finished, report saved to: {report_path}")
            return report_path
        return None

    def save_report(self) -> Optional[str]:
        """
        手动保存报告

        Returns:
            报告文件路径
        """
        if self.session_recorder:
            return self.session_recorder.save_report()
        return None

    def get_report_dir(self) -> Optional[str]:
        """获取报告目录路径"""
        if self.session_recorder:
            return str(self.session_recorder.run_manager.report_dir)
        return None

    async def __aenter__(self):
        """异步上下文管理器入口"""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器退出 - 自动保存报告"""
        if exc_type and self.session_recorder:
            # 如果有异常，标记当前步骤失败
            if self.session_recorder.current_step:
                self.session_recorder.fail_step(str(exc_val))

        self.finish()
        return False


__all__ = ["Agent"]



"""
任务缓存系统 - 对应 packages/core/src/agent/task-cache.ts

提供基于 YAML 的缓存系统，用于缓存 AI 的规划和定位结果。
"""

from typing import Optional, Dict, Any, List, Callable, Literal, Set
from dataclasses import dataclass, field
from pathlib import Path
import yaml
import json

from ...shared.logger import logger
from ...shared.utils import calculate_hash


# 缓存类型定义
CacheType = Literal["plan", "locate"]
CacheStrategy = Literal["read-only", "read-write", "write-only"]


@dataclass
class PlanningCache:
    """规划缓存"""
    type: Literal["plan"] = "plan"
    prompt: str = ""
    yaml_workflow: str = ""


@dataclass
class LocateCache:
    """定位缓存"""
    type: Literal["locate"] = "locate"
    prompt: str | Dict[str, Any] = ""
    cache: Optional[Dict[str, Any]] = None


@dataclass
class MatchCacheResult:
    """缓存匹配结果"""
    cache_content: PlanningCache | LocateCache
    update_fn: Callable[[PlanningCache | LocateCache], None]


@dataclass
class CacheFileContent:
    """缓存文件内容"""
    midscene_version: str  # 与 JS 版本对齐：midsceneVersion
    cache_id: str
    caches: List[PlanningCache | LocateCache] = field(default_factory=list)


class TaskCache:
    """任务缓存管理器"""

    CACHE_FILE_EXT = ".cache.yaml"
    DEFAULT_CACHE_MAX_FILENAME_LENGTH = 200
    MIDSCENE_VERSION = "1.0.0"  # 与 JS 版本对齐

    def __init__(
        self,
        cache_id: str,
        is_cache_result_used: bool = True,
        cache_file_path: Optional[str] = None,
        strategy: CacheStrategy = "read-write",
        cache_dir: Optional[str] = None,
    ):
        """
        初始化任务缓存

        Args:
            cache_id: 缓存 ID（通常是测试名称）
            is_cache_result_used: 是否使用缓存结果
            cache_file_path: 缓存文件路径（可选）
            strategy: 缓存策略（read-only, read-write, write-only）
            cache_dir: 缓存目录（可选）
        """
        if not cache_id:
            raise ValueError("cache_id is required")

        # 清理缓存 ID（移除非法字符）
        safe_cache_id = self._sanitize_cache_id(cache_id)

        # 限制文件名长度
        if len(safe_cache_id.encode('utf-8')) > self.DEFAULT_CACHE_MAX_FILENAME_LENGTH:
            prefix = safe_cache_id[:32]
            hash_suffix = calculate_hash(safe_cache_id)[:8]
            safe_cache_id = f"{prefix}-{hash_suffix}"

        self.cache_id = safe_cache_id

        # 设置缓存策略
        self.read_only_mode = strategy == "read-only"
        self.write_only_mode = strategy == "write-only"

        if self.read_only_mode and self.write_only_mode:
            raise ValueError("TaskCache cannot be both read-only and write-only")

        # write-only 模式下不使用缓存结果
        self.is_cache_result_used = False if self.write_only_mode else is_cache_result_used

        # 确定缓存文件路径（与 JS 版本对齐：midscene_run/cache/）
        if cache_file_path:
            self.cache_file_path = Path(cache_file_path)
        else:
            if cache_dir is None:
                # 与 JS 版本保持一致，使用 midscene_run/cache 目录
                cache_dir = "./midscene_run/cache"
            cache_dir_path = Path(cache_dir)
            cache_dir_path.mkdir(parents=True, exist_ok=True)
            self.cache_file_path = cache_dir_path / f"{self.cache_id}{self.CACHE_FILE_EXT}"

        # 加载或初始化缓存
        cache_content: Optional[CacheFileContent] = None
        if not self.write_only_mode:
            cache_content = self._load_cache_from_file()

        if cache_content is None:
            cache_content = CacheFileContent(
                midscene_version=self.MIDSCENE_VERSION,
                cache_id=self.cache_id,
                caches=[]
            )

        self.cache = cache_content
        self.cache_original_length = len(self.cache.caches) if self.is_cache_result_used else 0

        # 跟踪已匹配的缓存记录
        self.matched_cache_indices: Set[str] = set()

        logger.debug(
            f"TaskCache initialized: id={self.cache_id}, "
            f"strategy={strategy}, "
            f"records={len(self.cache.caches)}, "
            f"path={self.cache_file_path}"
        )

    def _sanitize_cache_id(self, cache_id: str) -> str:
        """清理缓存 ID，移除非法字符"""
        # 替换非法路径字符
        illegal_chars = '<>:"|?*\\/\n\r\t'
        safe_id = cache_id
        for char in illegal_chars:
            safe_id = safe_id.replace(char, "_")
        # 替换空格
        safe_id = safe_id.replace(" ", "-")
        return safe_id

    def match_cache(
        self,
        prompt: str | Dict[str, Any],
        cache_type: CacheType
    ) -> Optional[MatchCacheResult]:
        """
        匹配缓存

        Args:
            prompt: Prompt（字符串或字典）
            cache_type: 缓存类型（plan 或 locate）

        Returns:
            匹配结果或 None
        """
        if not self.is_cache_result_used:
            return None

        # 将 prompt 转换为字符串用于比较
        prompt_str = prompt if isinstance(prompt, str) else json.dumps(prompt, sort_keys=True)

        # 查找第一个未使用的匹配缓存
        for i in range(self.cache_original_length):
            item = self.cache.caches[i]
            key = f"{cache_type}:{prompt_str}:{i}"

            # 检查类型和 prompt 是否匹配，且未被使用
            item_prompt_str = (
                item.prompt if isinstance(item.prompt, str)
                else json.dumps(item.prompt, sort_keys=True)
            )

            if (
                item.type == cache_type
                and item_prompt_str == prompt_str
                and key not in self.matched_cache_indices
            ):
                # 标记为已使用
                self.matched_cache_indices.add(key)

                logger.debug(
                    f"Cache found: type={cache_type}, "
                    f"prompt={prompt_str[:50]}..., "
                    f"index={i}"
                )

                # 创建更新函数
                def update_fn(cache_item: PlanningCache | LocateCache) -> None:
                    """更新缓存项"""
                    logger.debug(
                        f"Updating cache: type={cache_type}, "
                        f"prompt={prompt_str[:50]}..., "
                        f"index={i}"
                    )

                    # 更新缓存内容
                    self.cache.caches[i] = cache_item

                    if self.read_only_mode:
                        logger.debug("Read-only mode: cache updated in memory only")
                        return

                    # 写入文件
                    self._flush_cache_to_file()

                return MatchCacheResult(
                    cache_content=item,
                    update_fn=update_fn
                )

        logger.debug(f"No cache found: type={cache_type}, prompt={prompt_str[:50]}...")
        return None

    def match_plan_cache(self, prompt: str) -> Optional[MatchCacheResult]:
        """匹配规划缓存"""
        return self.match_cache(prompt, "plan")

    def match_locate_cache(
        self,
        prompt: str | Dict[str, Any]
    ) -> Optional[MatchCacheResult]:
        """匹配定位缓存"""
        return self.match_cache(prompt, "locate")

    def append_cache(self, cache: PlanningCache | LocateCache) -> None:
        """
        添加新缓存

        Args:
            cache: 缓存项
        """
        logger.debug(f"Appending cache: type={cache.type}")
        self.cache.caches.append(cache)

        if self.read_only_mode:
            logger.debug("Read-only mode: cache appended to memory only")
            return

        self._flush_cache_to_file()

    def _load_cache_from_file(self) -> Optional[CacheFileContent]:
        """从文件加载缓存"""
        if not self.cache_file_path.exists():
            logger.debug(f"No cache file found: {self.cache_file_path}")
            return None

        try:
            with open(self.cache_file_path, 'r', encoding='utf-8') as f:
                data = yaml.safe_load(f)

            if not data:
                return None

            # 转换为 CacheFileContent（兼容 JS 版本的 midsceneVersion 和旧版 pymidsceneVersion）
            cache_content = CacheFileContent(
                midscene_version=data.get("midsceneVersion", data.get("pymidsceneVersion", "0.0.0")),
                cache_id=data.get("cacheId", self.cache_id),
                caches=[]
            )

            # 解析缓存项
            for item in data.get("caches", []):
                if item.get("type") == "plan":
                    cache_content.caches.append(PlanningCache(
                        type="plan",
                        prompt=item.get("prompt", ""),
                        yaml_workflow=item.get("yamlWorkflow", "")
                    ))
                elif item.get("type") == "locate":
                    cache_content.caches.append(LocateCache(
                        type="locate",
                        prompt=item.get("prompt", ""),
                        cache=item.get("cache")
                    ))

            logger.info(
                f"Cache loaded: {self.cache_file_path}, "
                f"version={cache_content.midscene_version}, "
                f"records={len(cache_content.caches)}"
            )

            # 更新版本号
            cache_content.midscene_version = self.MIDSCENE_VERSION

            return cache_content

        except Exception as e:
            logger.error(f"Failed to load cache file: {self.cache_file_path}, error: {e}")
            return None

    def _flush_cache_to_file(self, clean_unused: bool = False) -> None:
        """
        将缓存写入文件

        Args:
            clean_unused: 是否清理未使用的缓存
        """
        if not self.cache_file_path:
            logger.debug("No cache file path, will not write cache")
            return

        # 清理未使用的缓存
        if clean_unused and self.is_cache_result_used:
            original_length = len(self.cache.caches)

            # 收集已使用的索引
            used_indices = set()
            for key in self.matched_cache_indices:
                # key 格式: "type:prompt:index"
                parts = key.split(":")
                try:
                    index = int(parts[-1])
                    used_indices.add(index)
                except (ValueError, IndexError):
                    pass

            # 过滤：保留已使用的缓存和新添加的缓存
            self.cache.caches = [
                cache for i, cache in enumerate(self.cache.caches)
                if i in used_indices or i >= self.cache_original_length
            ]

            cleaned_count = original_length - len(self.cache.caches)
            if cleaned_count > 0:
                logger.info(f"Cleaned {cleaned_count} unused cache records")

        # 确保目录存在
        self.cache_file_path.parent.mkdir(parents=True, exist_ok=True)

        # 转换为可序列化的字典（与 JS 版本对齐：midsceneVersion）
        data = {
            "midsceneVersion": self.cache.midscene_version,
            "cacheId": self.cache.cache_id,
            "caches": []
        }

        for cache_item in self.cache.caches:
            if isinstance(cache_item, PlanningCache):
                data["caches"].append({
                    "type": "plan",
                    "prompt": cache_item.prompt,
                    "yamlWorkflow": cache_item.yaml_workflow
                })
            elif isinstance(cache_item, LocateCache):
                data["caches"].append({
                    "type": "locate",
                    "prompt": cache_item.prompt,
                    "cache": cache_item.cache
                })

        # 写入 YAML 文件
        try:
            with open(self.cache_file_path, 'w', encoding='utf-8') as f:
                yaml.dump(
                    data,
                    f,
                    default_flow_style=False,
                    allow_unicode=True,
                    sort_keys=False
                )

            logger.debug(
                f"Cache flushed to file: {self.cache_file_path}, "
                f"records={len(self.cache.caches)}"
            )

        except Exception as e:
            logger.error(f"Failed to write cache file: {self.cache_file_path}, error: {e}")

    def get_stats(self) -> Dict[str, Any]:
        """获取缓存统计信息"""
        return {
            "cache_id": self.cache_id,
            "total_records": len(self.cache.caches),
            "original_records": self.cache_original_length,
            "matched_records": len(self.matched_cache_indices),
            "strategy": (
                "read-only" if self.read_only_mode
                else "write-only" if self.write_only_mode
                else "read-write"
            ),
            "cache_file": str(self.cache_file_path),
        }

    def __repr__(self) -> str:
        stats = self.get_stats()
        return (
            f"TaskCache(id={stats['cache_id']}, "
            f"records={stats['total_records']}, "
            f"matched={stats['matched_records']}, "
            f"strategy={stats['strategy']})"
        )


__all__ = [
    "TaskCache",
    "PlanningCache",
    "LocateCache",
    "MatchCacheResult",
    "CacheFileContent",
    "CacheType",
    "CacheStrategy",
]



"""
执行记录系统 (Dump) - 对应 packages/core/src/dump/

这个模块提供执行过程的记录和序列化功能，用于：
- 记录每步执行的截图、AI 调用信息、耗时
- 生成 JSON 格式的执行报告
- 生成 HTML 可视化报告
- 与 JS 版本的 Visualizer 保持兼容
"""

import json
import time
import uuid
from datetime import datetime
from typing import Dict, Any, List, Optional, Tuple
from dataclasses import asdict

from .types import (
    ExecutionDump,
    GroupedActionDump,
    ExecutionTask,
    ExecutionRecorderItem,
    ServiceDump,
    ScreenshotItem,
    AIUsageInfo,
)
from .run_manager import MidsceneRunManager, get_default_run_manager
from .element_marker import ElementMarker, get_default_marker, ActionMarker
from .report_generator import (
    HTMLReportGenerator,
    ReportSession,
    ReportStep,
    get_default_report_generator,
)
from .js_react_report_generator import (
    JSReactReportGenerator,
    get_js_react_report_generator,
)
from ..shared.logger import logger
from ..shared.types import LocateResultElement


class ExecutionRecorder:
    """执行记录器 - 记录单次执行的详细信息"""

    def __init__(self, name: str, description: Optional[str] = None):
        """
        初始化执行记录器

        Args:
            name: 执行名称
            description: 执行描述
        """
        self.name = name
        self.description = description
        self.log_time = time.time()
        self.tasks: List[ExecutionTask] = []
        self.current_task: Optional[ExecutionTask] = None

    def start_task(
        self,
        task_type: str,
        param: Any = None,
        thought: Optional[str] = None
    ) -> ExecutionTask:
        """
        开始记录新任务

        Args:
            task_type: 任务类型 ('locate', 'click', 'input', 'query', 'assert')
            param: 任务参数
            thought: AI 思考内容

        Returns:
            创建的任务对象
        """
        from .types import UIContext
        from typing import cast
        from ..shared.types import ExecutionTaskType, TaskStatus

        task = ExecutionTask(
            type=cast(ExecutionTaskType, task_type),
            param=param,
            thought=thought,
            status=cast(TaskStatus, "pending"),
            recorder=[],
        )

        self.tasks.append(task)
        self.current_task = task

        logger.debug(f"开始记录任务: {task_type}, 参数: {param}")
        return task

    def record_screenshot(self, screenshot: ScreenshotItem, timing: str = "before"):
        """
        记录截图

        Args:
            screenshot: 截图对象
            timing: 时机 ('before', 'after')
        """
        if not self.current_task:
            logger.warning("没有当前任务，无法记录截图")
            return

        record_item = ExecutionRecorderItem(
            type="screenshot",
            ts=time.time(),
            screenshot=screenshot,
            timing=timing
        )

        self.current_task.recorder.append(record_item)
        logger.debug(f"记录截图: {timing}")

    def record_ai_usage(self, usage: AIUsageInfo):
        """
        记录 AI 使用信息

        Args:
            usage: AI 使用信息（tokens, 耗时等）- 可以是 AIUsageInfo 对象或字典
        """
        if not self.current_task:
            logger.warning("没有当前任务，无法记录 AI 使用信息")
            return

        self.current_task.usage = usage
        # 兼容字典和对象两种格式
        if isinstance(usage, dict):
            tokens = usage.get('total_tokens', 'N/A')
        else:
            tokens = usage.total_tokens if usage else 'N/A'
        logger.debug(f"记录 AI 使用: {tokens} tokens")

    def finish_task(
        self,
        status: str = "finished",
        output: Any = None,
        error: Optional[Exception] = None
    ):
        """
        完成当前任务

        Args:
            status: 任务状态 ('finished', 'failed')
            output: 任务输出
            error: 错误信息（如果失败）
        """
        if not self.current_task:
            logger.warning("没有当前任务，无法完成")
            return

        from typing import cast
        from ..shared.types import TaskStatus
        self.current_task.status = cast(TaskStatus, status)
        self.current_task.output = output

        if error:
            self.current_task.error = error
            self.current_task.error_message = str(error)

        logger.debug(f"完成任务: {self.current_task.type}, 状态: {status}")
        self.current_task = None

    def to_dump(self) -> ExecutionDump:
        """
        转换为 ExecutionDump 对象

        Returns:
            ExecutionDump 对象
        """
        return ExecutionDump(
            log_time=self.log_time,
            name=self.name,
            description=self.description,
            tasks=self.tasks,
        )

    def to_json(self) -> str:
        """
        导出为 JSON 字符串

        Returns:
            JSON 字符串
        """
        dump = self.to_dump()
        return json.dumps(dump.to_dict(), indent=2, ensure_ascii=False)


class SessionRecorder:
    """
    会话记录器 - 记录完整的自动化会话

    集成了目录管理、元素标记、HTML报告生成等功能。
    与 JS 版本 Midscene 的日志系统对齐。
    """

    def __init__(
        self,
        driver_type: str = "playwright",
        base_dir: Optional[str] = None,
        auto_save: bool = True,
        session_id: Optional[str] = None,
        use_js_react_report: bool = True,  # 默认使用 JS React 报告生成器
    ):
        """
        初始化会话记录器

        Args:
            driver_type: 驱动类型 ('playwright', 'selenium', 'appium')
            base_dir: 基础目录，默认为当前工作目录
            auto_save: 是否自动保存报告
            session_id: 会话ID，默认自动生成
            use_js_react_report: 是否使用 JS React 报告生成器（与 JS 版本视觉一致）
        """
        self.driver_type = driver_type
        self.auto_save = auto_save
        self.session_id = session_id or uuid.uuid4().hex
        self.use_js_react_report = use_js_react_report

        # 初始化组件
        self.run_manager = MidsceneRunManager(base_dir)
        self.element_marker = get_default_marker()
        self.report_generator = get_default_report_generator()
        
        # JS React 报告生成器（当 use_js_react_report=True 时使用）
        self.js_react_generator: Optional[JSReactReportGenerator] = None
        if use_js_react_report:
            self.js_react_generator = get_js_react_report_generator()

        # 会话数据
        self.start_time = datetime.now()
        self.end_time: Optional[datetime] = None
        self.steps: List[ReportStep] = []
        self.current_step: Optional[ReportStep] = None
        self._step_index = 0
        self._step_start_time: float = 0.0

        # 元数据
        self.page_url: Optional[str] = None
        self.page_title: Optional[str] = None
        self.viewport_size: Optional[Dict[str, int]] = None

        logger.info(f"SessionRecorder initialized: {self.session_id}")

    def set_page_info(
        self,
        url: Optional[str] = None,
        title: Optional[str] = None,
        viewport: Optional[Dict[str, int]] = None
    ):
        """设置页面信息"""
        if url:
            self.page_url = url
        if title:
            self.page_title = title
        if viewport:
            self.viewport_size = viewport

    def start_step(
        self,
        action_type: str,
        prompt: str,
    ) -> ReportStep:
        """
        开始记录新步骤

        Args:
            action_type: 操作类型 ('click', 'input', 'scroll', 'assert', 'query', 'locate')
            prompt: 用户指令/描述

        Returns:
            创建的步骤对象
        """
        self._step_index += 1

        step = ReportStep(
            step_id=uuid.uuid4().hex[:8],
            step_index=self._step_index,
            action_type=action_type,
            prompt=prompt,
            timestamp=datetime.now().isoformat(),
            duration_ms=0,
            status="pending",
        )

        self.steps.append(step)
        self.current_step = step
        self._step_start_time = time.time()

        logger.debug(f"开始步骤 #{self._step_index}: {action_type} - {prompt[:50]}...")
        return step

    def record_screenshot_before(self, screenshot_base64: str):
        """记录操作前截图"""
        if self.current_step:
            self.current_step.screenshot_before = screenshot_base64

    def record_screenshot_after(self, screenshot_base64: str):
        """记录操作后截图"""
        if self.current_step:
            self.current_step.screenshot_after = screenshot_base64

    def record_element_location(
        self,
        bbox: Tuple[int, int, int, int],
        center: Tuple[int, int],
        description: Optional[str] = None,
        draw_marker: bool = True
    ):
        """
        记录元素定位结果

        Args:
            bbox: 边界框 (x1, y1, x2, y2)
            center: 中心点 (x, y)
            description: 元素描述
            draw_marker: 是否在截图上绘制标记
        """
        if not self.current_step:
            return

        self.current_step.element_bbox = list(bbox)
        self.current_step.element_center = list(center)
        self.current_step.element_description = description

        # 在截图上绘制标记
        if draw_marker and self.current_step.screenshot_before:
            marked = self.element_marker.draw_element_with_click(
                self.current_step.screenshot_before,
                bbox,
                center,
                label=description
            )
            self.current_step.screenshot_marked = marked

    def record_ai_info(
        self,
        model: Optional[str] = None,
        tokens: Optional[int] = None,
        response: Optional[str] = None,
        reasoning: Optional[str] = None
    ):
        """记录 AI 调用信息"""
        if not self.current_step:
            return

        if model:
            self.current_step.ai_model = model
        if tokens:
            self.current_step.ai_tokens = tokens
        if response:
            self.current_step.ai_response = response
        if reasoning:
            self.current_step.ai_reasoning = reasoning

    def complete_step(
        self,
        status: str = "success",
        error_message: Optional[str] = None
    ):
        """
        完成当前步骤

        Args:
            status: 状态 ('success', 'failed')
            error_message: 错误信息
        """
        if not self.current_step:
            return

        # 计算耗时
        duration = int((time.time() - self._step_start_time) * 1000)
        self.current_step.duration_ms = duration
        self.current_step.status = status

        if error_message:
            self.current_step.error_message = error_message

        logger.debug(
            f"完成步骤 #{self.current_step.step_index}: "
            f"{status}, {duration}ms"
        )

        self.current_step = None

    def fail_step(self, error_message: str):
        """标记当前步骤失败"""
        self.complete_step(status="failed", error_message=error_message)

    def _build_report_session(self) -> ReportSession:
        """构建报告会话对象"""
        success_count = sum(1 for s in self.steps if s.status == "success")
        failed_count = sum(1 for s in self.steps if s.status == "failed")

        overall_status = "success"
        if failed_count > 0:
            overall_status = "failed"
        elif not self.end_time:
            overall_status = "running"

        return ReportSession(
            session_id=self.session_id,
            driver_type=self.driver_type,
            start_time=self.start_time.isoformat(),
            end_time=self.end_time.isoformat() if self.end_time else None,
            status=overall_status,
            total_steps=len(self.steps),
            success_steps=success_count,
            failed_steps=failed_count,
            steps=self.steps,
            page_url=self.page_url,
            page_title=self.page_title,
            viewport_size=self.viewport_size,
        )

    def generate_report(self) -> str:
        """
        生成 HTML 报告

        Returns:
            HTML 内容
        """
        # 优先使用 JS React 报告生成器
        if self.use_js_react_report and self.js_react_generator:
            return self._generate_js_react_report()
        
        session = self._build_report_session()
        return self.report_generator.generate(session)
    
    def _generate_js_react_report(self) -> str:
        """使用 JS React 报告生成器生成报告"""
        if not self.js_react_generator:
            raise RuntimeError("JS React generator not initialized")
        
        # 初始化会话
        self.js_react_generator.start_session(
            group_name=f"PyMidscene Session - {self.session_id}",
            description=f"Driver: {self.driver_type}",
        )
        
        # 将步骤转换为 JS React 格式
        for step in self.steps:
            # 映射操作类型到 JS 版本格式
            task_type = "Insight"
            sub_type = step.action_type.capitalize()
            
            if step.action_type in ["click", "tap", "input", "scroll"]:
                task_type = "Action Space"
                sub_type = "Tap" if step.action_type in ["click", "tap"] else step.action_type.capitalize()
            elif step.action_type in ["locate"]:
                task_type = "Planning"
                sub_type = "Locate"
            elif step.action_type in ["assert", "query"]:
                task_type = "Insight"
                sub_type = step.action_type.capitalize()
            
            # 转换元素信息
            element_rect = None
            element_center = None
            if step.element_bbox and len(step.element_bbox) == 4:
                element_rect = {
                    "left": step.element_bbox[0],
                    "top": step.element_bbox[1],
                    "width": step.element_bbox[2] - step.element_bbox[0],
                    "height": step.element_bbox[3] - step.element_bbox[1],
                }
            if step.element_center:
                element_center = step.element_center
            
            # 添加任务
            self.js_react_generator.add_task(
                task_type=task_type,
                sub_type=sub_type,
                prompt=step.prompt,
                status="finished" if step.status == "success" else "failed" if step.status == "failed" else "pending",
                screenshot_before=step.screenshot_before,
                screenshot_after=step.screenshot_after,
                element_rect=element_rect,
                element_center=element_center,
                element_text=step.element_description,
                duration_ms=step.duration_ms,
                ai_tokens=step.ai_tokens,
                error=step.error_message,
                thought=step.ai_reasoning,
            )
        
        return self.js_react_generator.generate_html()

    def save_report(self, filename: Optional[str] = None) -> str:
        """
        保存 HTML 报告到文件

        Args:
            filename: 文件名（可选）

        Returns:
            保存的文件路径
        """
        # 优先使用 JS React 报告生成器
        if self.use_js_react_report and self.js_react_generator:
            return self._save_js_react_report(filename)
        
        session = self._build_report_session()
        return self.report_generator.save(
            session,
            str(self.run_manager.report_dir),
            filename
        )
    
    def _save_js_react_report(self, filename: Optional[str] = None) -> str:
        """使用 JS React 报告生成器保存报告"""
        # 先生成报告内容（会初始化会话和添加任务）
        self._generate_js_react_report()
        
        if not self.js_react_generator:
            raise RuntimeError("JS React generator not initialized")
        
        # 保存报告
        return self.js_react_generator.save(
            str(self.run_manager.report_dir),
            self.driver_type,
            filename
        )

    def save_dump(self) -> str:
        """
        保存 JSON 转储文件

        Returns:
            保存的文件路径
        """
        session = self._build_report_session()
        dump_path = self.run_manager.get_dump_file_path(self.session_id)

        with open(dump_path, 'w', encoding='utf-8') as f:
            json.dump(session.to_dict(), f, indent=2, ensure_ascii=False)

        logger.info(f"Dump saved to: {dump_path}")
        return str(dump_path)

    def finish(self) -> Optional[str]:
        """
        结束会话并保存报告

        Returns:
            报告文件路径（如果 auto_save=True）
        """
        self.end_time = datetime.now()

        logger.info(
            f"Session finished: {self.session_id}, "
            f"steps={len(self.steps)}, "
            f"duration={(self.end_time - self.start_time).total_seconds():.1f}s"
        )

        if self.auto_save:
            return self.save_report()

        return None

    def __enter__(self):
        """上下文管理器入口"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器退出"""
        if exc_type:
            # 如果有异常，标记当前步骤失败
            if self.current_step:
                self.fail_step(str(exc_val))

        self.finish()
        return False


class GroupedExecutionRecorder:
    """分组执行记录器 - 管理多个执行记录"""

    def __init__(
        self,
        group_name: str,
        group_description: Optional[str] = None,
        sdk_version: str = "1.0.0"
    ):
        """
        初始化分组记录器

        Args:
            group_name: 分组名称
            group_description: 分组描述
            sdk_version: SDK 版本
        """
        self.group_name = group_name
        self.group_description = group_description
        self.sdk_version = sdk_version
        self.executions: List[ExecutionDump] = []
        self.model_briefs: List[str] = []

    def add_execution(self, execution: ExecutionDump):
        """
        添加执行记录

        Args:
            execution: 执行记录对象
        """
        self.executions.append(execution)
        logger.debug(f"添加执行记录: {execution.name}")

    def add_model_brief(self, model_name: str):
        """
        添加模型简介

        Args:
            model_name: 模型名称
        """
        if model_name not in self.model_briefs:
            self.model_briefs.append(model_name)

    def to_dump(self) -> GroupedActionDump:
        """
        转换为 GroupedActionDump 对象

        Returns:
            GroupedActionDump 对象
        """
        return GroupedActionDump(
            sdk_version=self.sdk_version,
            group_name=self.group_name,
            group_description=self.group_description,
            model_briefs=self.model_briefs,
            executions=self.executions,
        )

    def to_json(self) -> str:
        """
        导出为 JSON 字符串

        Returns:
            JSON 字符串
        """
        dump = self.to_dump()
        return json.dumps(dump.to_dict(), indent=2, ensure_ascii=False)

    def save_to_file(self, file_path: str):
        """
        保存到文件

        Args:
            file_path: 文件路径
        """
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(self.to_json())

        logger.info(f"执行记录已保存到: {file_path}")

    @staticmethod
    def load_from_file(file_path: str) -> 'GroupedExecutionRecorder':
        """
        从文件加载

        Args:
            file_path: 文件路径

        Returns:
            GroupedExecutionRecorder 对象
        """
        with open(file_path, 'r', encoding='utf-8') as f:
            data = json.load(f)

        dump = GroupedActionDump.from_dict(data)

        recorder = GroupedExecutionRecorder(
            group_name=dump.group_name,
            group_description=dump.group_description,
            sdk_version=dump.sdk_version
        )
        recorder.executions = dump.executions
        recorder.model_briefs = dump.model_briefs

        logger.info(f"执行记录已从文件加载: {file_path}")
        return recorder


class ServiceDumpBuilder:
    """ServiceDump 构建器 - 用于构建 AI 服务调用的记录"""

    def __init__(self, dump_type: str):
        """
        初始化构建器

        Args:
            dump_type: 记录类型 ('locate', 'extract', 'assert')
        """
        self.dump_type = dump_type
        self.log_id = str(uuid.uuid4())
        self.log_time = time.time()
        self.user_query: Dict[str, Any] = {}
        self.matched_element: List[LocateResultElement] = []
        self.data: Any = None
        self.task_info: Dict[str, Any] = {
            "durationMs": 0,
        }
        self.error: Optional[str] = None

    def set_user_query(self, **kwargs):
        """设置用户查询"""
        self.user_query.update(kwargs)
        return self

    def set_matched_element(self, elements: List[LocateResultElement]):
        """设置匹配的元素"""
        self.matched_element = elements
        return self

    def set_data(self, data: Any):
        """设置数据"""
        self.data = data
        return self

    def set_task_info(self, **kwargs):
        """设置任务信息"""
        self.task_info.update(kwargs)
        return self

    def set_error(self, error: str):
        """设置错误信息"""
        self.error = error
        return self

    def build(self) -> ServiceDump:
        """
        构建 ServiceDump 对象

        Returns:
            ServiceDump 对象
        """
        from .types import ServiceTaskInfo

        task_info = ServiceTaskInfo(
            duration_ms=self.task_info.get("durationMs", 0),
            format_response=self.task_info.get("formatResponse"),
            raw_response=self.task_info.get("rawResponse"),
            usage=self.task_info.get("usage"),
            search_area=self.task_info.get("searchArea"),
            reasoning_content=self.task_info.get("reasoningContent"),
        )

        return ServiceDump(
            type=self.dump_type,
            log_id=self.log_id,
            log_time=self.log_time,
            user_query=self.user_query,
            matched_element=self.matched_element,
            matched_rect=None,
            deep_think=None,
            data=self.data,
            assertion_pass=None,
            assertion_thought=None,
            task_info=task_info,
            error=self.error,
            output=None,
        )


def create_execution_recorder(
    name: str,
    description: Optional[str] = None
) -> ExecutionRecorder:
    """
    创建执行记录器的便捷函数

    Args:
        name: 执行名称
        description: 执行描述

    Returns:
        ExecutionRecorder 对象
    """
    return ExecutionRecorder(name, description)


def create_session_recorder(
    driver_type: str = "playwright",
    base_dir: Optional[str] = None,
    auto_save: bool = True,
    use_js_react_report: bool = True,
) -> SessionRecorder:
    """
    创建会话记录器的便捷函数

    Args:
        driver_type: 驱动类型
        base_dir: 基础目录
        auto_save: 是否自动保存
        use_js_react_report: 是否使用 JS React 报告生成器（与 JS 版本视觉一致）

    Returns:
        SessionRecorder 对象
    """
    return SessionRecorder(driver_type, base_dir, auto_save, use_js_react_report=use_js_react_report)


def create_grouped_recorder(
    group_name: str,
    group_description: Optional[str] = None
) -> GroupedExecutionRecorder:
    """
    创建分组记录器的便捷函数

    Args:
        group_name: 分组名称
        group_description: 分组描述

    Returns:
        GroupedExecutionRecorder 对象
    """
    return GroupedExecutionRecorder(group_name, group_description)


__all__ = [
    "ExecutionRecorder",
    "SessionRecorder",
    "GroupedExecutionRecorder",
    "ServiceDumpBuilder",
    "create_execution_recorder",
    "create_session_recorder",
    "create_grouped_recorder",
]



"""
抽象接口定义 - 对应 packages/core/src/device/

定义设备交互的统一接口，支持 Web、Android、iOS 等平台。
"""

from typing import Optional, Dict, Any, List, Protocol
from abc import ABC, abstractmethod

from ..core.types import UIContext
from ..shared.types import Size


class AbstractInterface(ABC):
    """设备交互抽象接口"""

    @abstractmethod
    async def get_ui_context(self) -> UIContext:
        """
        获取当前 UI 上下文（包括截图和尺寸）

        Returns:
            UIContext 实例
        """
        pass

    @abstractmethod
    async def get_size(self) -> Size:
        """
        获取视口尺寸

        Returns:
            Size 字典
        """
        pass

    @abstractmethod
    async def screenshot(self, full_page: bool = False) -> str:
        """
        获取截图（Base64 编码）

        Args:
            full_page: 是否截取完整页面

        Returns:
            Base64 编码的图像字符串
        """
        pass

    @abstractmethod
    async def click(self, x: float, y: float) -> None:
        """
        点击指定坐标

        Args:
            x: X 坐标
            y: Y 坐标
        """
        pass

    @abstractmethod
    async def input_text(self, text: str, x: Optional[float] = None, y: Optional[float] = None) -> None:
        """
        输入文本

        Args:
            text: 要输入的文本
            x: 可选的 X 坐标（先点击再输入）
            y: 可选的 Y 坐标（先点击再输入）
        """
        pass

    @abstractmethod
    async def hover(self, x: float, y: float) -> None:
        """
        悬停到指定坐标

        Args:
            x: X 坐标
            y: Y 坐标
        """
        pass

    @abstractmethod
    async def scroll(self, direction: str, distance: Optional[int] = None) -> None:
        """
        滚动页面

        Args:
            direction: 滚动方向（up/down/left/right）
            distance: 滚动距离（像素）
        """
        pass

    @abstractmethod
    async def key_press(self, key: str) -> None:
        """
        按键

        Args:
            key: 按键名称（Enter、Escape 等）
        """
        pass

    @abstractmethod
    async def wait_for_navigation(self, timeout: Optional[int] = None) -> None:
        """
        等待导航完成

        Args:
            timeout: 超时时间（毫秒）
        """
        pass

    @abstractmethod
    async def wait_for_network_idle(self, timeout: Optional[int] = None) -> None:
        """
        等待网络空闲

        Args:
            timeout: 超时时间（毫秒）
        """
        pass

    @abstractmethod
    async def evaluate_javascript(self, script: str) -> Any:
        """
        执行 JavaScript 代码

        Args:
            script: JavaScript 代码

        Returns:
            执行结果
        """
        pass

    # ==================== XPath 相关方法（与 JS 版本对齐） ====================

    async def get_element_xpath(self, x: float, y: float) -> Optional[str]:
        """
        获取指定坐标处元素的 XPath

        Args:
            x: X 坐标
            y: Y 坐标

        Returns:
            元素的 XPath 路径，如果找不到则返回 None
        """
        # 默认实现，子类可以覆盖
        return None

    async def get_element_by_xpath(self, xpath: str) -> Optional[Dict[str, Any]]:
        """
        通过 XPath 获取元素信息（边界框和中心点）

        Args:
            xpath: 元素的 XPath 路径

        Returns:
            包含 bbox, center, rect 的字典，如果找不到则返回 None
        """
        # 默认实现，子类可以覆盖
        return None


__all__ = ["AbstractInterface"]



"""
元素标记器 - 在截图上绘制元素边界框、点击位置等标记

用于生成可视化报告，标记 AI 识别到的元素和执行的操作。
"""

import base64
import io
from typing import Tuple, List, Optional, Union
from dataclasses import dataclass

try:
    from PIL import Image, ImageDraw, ImageFont
    HAS_PIL = True
except ImportError:
    HAS_PIL = False

from ..shared.logger import logger


@dataclass
class MarkerStyle:
    """标记样式配置"""
    # 边界框样式
    bbox_color: str = "#FF0000"  # 红色
    bbox_width: int = 3
    bbox_fill: Optional[str] = None  # 填充颜色（带透明度）

    # 点击标记样式
    click_color: str = "#00FF00"  # 绿色
    click_radius: int = 15
    click_cross_size: int = 20

    # 标签样式
    label_color: str = "#FFFFFF"  # 白色文字
    label_bg_color: str = "#FF0000"  # 红色背景
    label_font_size: int = 14
    label_padding: int = 4

    # 序号标记样式
    index_color: str = "#FFFFFF"
    index_bg_color: str = "#0066FF"  # 蓝色背景
    index_radius: int = 12


@dataclass
class ActionMarker:
    """操作标记"""
    action_type: str  # 'click', 'input', 'scroll', 'hover'
    point: Tuple[int, int]
    label: Optional[str] = None
    index: Optional[int] = None


class ElementMarker:
    """
    元素标记器

    在截图上绘制各种可视化标记：
    - 元素边界框 (bounding box)
    - 点击位置标记
    - 操作序列轨迹
    - 元素标签和序号
    """

    def __init__(self, style: Optional[MarkerStyle] = None):
        """
        初始化元素标记器

        Args:
            style: 标记样式配置
        """
        if not HAS_PIL:
            logger.warning(
                "Pillow not installed. ElementMarker will return original images. "
                "Install with: pip install Pillow"
            )

        self.style = style or MarkerStyle()
        self._font = None

    def _get_font(self, size: int = 14):
        """获取字体"""
        if not HAS_PIL:
            return None

        try:
            # 尝试加载系统字体
            return ImageFont.truetype("arial.ttf", size)
        except:
            try:
                # Windows 中文字体
                return ImageFont.truetype("msyh.ttc", size)
            except:
                try:
                    # macOS
                    return ImageFont.truetype("/System/Library/Fonts/PingFang.ttc", size)
                except:
                    # 使用默认字体
                    return ImageFont.load_default()

    def _parse_color(self, color: str) -> Tuple[int, int, int, int]:
        """
        解析颜色字符串为 RGBA

        Args:
            color: 颜色字符串 (#RGB, #RGBA, #RRGGBB, #RRGGBBAA)

        Returns:
            RGBA 元组
        """
        color = color.lstrip('#')

        if len(color) == 3:
            r, g, b = [int(c * 2, 16) for c in color]
            return (r, g, b, 255)
        elif len(color) == 4:
            r, g, b, a = [int(c * 2, 16) for c in color]
            return (r, g, b, a)
        elif len(color) == 6:
            r = int(color[0:2], 16)
            g = int(color[2:4], 16)
            b = int(color[4:6], 16)
            return (r, g, b, 255)
        elif len(color) == 8:
            r = int(color[0:2], 16)
            g = int(color[2:4], 16)
            b = int(color[4:6], 16)
            a = int(color[6:8], 16)
            return (r, g, b, a)
        else:
            return (255, 0, 0, 255)  # 默认红色

    def _base64_to_image(self, image_base64: str) -> Optional['Image.Image']:
        """将 base64 字符串转换为 PIL Image"""
        if not HAS_PIL:
            return None

        try:
            # 移除可能的 data URL 前缀
            if ',' in image_base64:
                image_base64 = image_base64.split(',')[1]

            image_data = base64.b64decode(image_base64)
            return Image.open(io.BytesIO(image_data)).convert('RGBA')
        except Exception as e:
            logger.error(f"Failed to decode base64 image: {e}")
            return None

    def _image_to_base64(self, image: 'Image.Image', format: str = 'PNG') -> str:
        """将 PIL Image 转换为 base64 字符串"""
        if not HAS_PIL:
            return ""

        buffer = io.BytesIO()
        # 转换为 RGB 模式以支持 JPEG
        if format.upper() == 'JPEG' and image.mode == 'RGBA':
            image = image.convert('RGB')
        image.save(buffer, format=format)
        return base64.b64encode(buffer.getvalue()).decode('utf-8')

    def draw_bbox(
        self,
        image_base64: str,
        bbox: Tuple[int, int, int, int],
        color: Optional[str] = None,
        label: Optional[str] = None,
        width: Optional[int] = None
    ) -> str:
        """
        在截图上绘制边界框

        Args:
            image_base64: 原始截图的 base64 字符串
            bbox: 边界框坐标 (x1, y1, x2, y2) 或 (x, y, width, height)
            color: 边框颜色
            label: 标签文字
            width: 边框宽度

        Returns:
            带标记的截图 base64 字符串
        """
        if not HAS_PIL:
            return image_base64

        image = self._base64_to_image(image_base64)
        if image is None:
            return image_base64

        draw = ImageDraw.Draw(image, 'RGBA')

        # 解析坐标
        x1, y1, x2_or_w, y2_or_h = bbox
        # 判断是 (x1, y1, x2, y2) 还是 (x, y, width, height)
        if x2_or_w > x1 and y2_or_h > y1:
            # 可能是 (x1, y1, x2, y2) 格式
            x2, y2 = x2_or_w, y2_or_h
        else:
            # (x, y, width, height) 格式
            x2 = x1 + x2_or_w
            y2 = y1 + y2_or_h

        # 绘制边界框
        box_color = self._parse_color(color or self.style.bbox_color)
        line_width = width or self.style.bbox_width

        # 绘制矩形边框
        draw.rectangle(
            [(x1, y1), (x2, y2)],
            outline=box_color[:3],
            width=line_width
        )

        # 如果有填充色，添加半透明填充
        if self.style.bbox_fill:
            fill_color = self._parse_color(self.style.bbox_fill)
            overlay = Image.new('RGBA', image.size, (0, 0, 0, 0))
            overlay_draw = ImageDraw.Draw(overlay)
            overlay_draw.rectangle(
                [(x1, y1), (x2, y2)],
                fill=fill_color
            )
            image = Image.alpha_composite(image, overlay)

        # 绘制标签
        if label:
            font = self._get_font(self.style.label_font_size)

            # 计算文字大小
            text_bbox = draw.textbbox((0, 0), label, font=font)
            text_width = text_bbox[2] - text_bbox[0]
            text_height = text_bbox[3] - text_bbox[1]

            padding = self.style.label_padding

            # 标签背景位置（在边界框上方）
            label_x = x1
            label_y = max(0, y1 - text_height - padding * 2 - 2)

            # 绘制标签背景
            bg_color = self._parse_color(self.style.label_bg_color)
            draw.rectangle(
                [
                    (label_x, label_y),
                    (label_x + text_width + padding * 2, label_y + text_height + padding * 2)
                ],
                fill=bg_color[:3]
            )

            # 绘制标签文字
            text_color = self._parse_color(self.style.label_color)
            draw.text(
                (label_x + padding, label_y + padding),
                label,
                fill=text_color[:3],
                font=font
            )

        return self._image_to_base64(image)

    def draw_click_point(
        self,
        image_base64: str,
        point: Tuple[int, int],
        color: Optional[str] = None,
        radius: Optional[int] = None
    ) -> str:
        """
        在截图上绘制点击位置标记

        Args:
            image_base64: 原始截图的 base64 字符串
            point: 点击位置坐标 (x, y)
            color: 标记颜色
            radius: 标记半径

        Returns:
            带标记的截图 base64 字符串
        """
        if not HAS_PIL:
            return image_base64

        image = self._base64_to_image(image_base64)
        if image is None:
            return image_base64

        draw = ImageDraw.Draw(image, 'RGBA')

        x, y = point
        click_color = self._parse_color(color or self.style.click_color)
        r = radius or self.style.click_radius
        cross_size = self.style.click_cross_size

        # 绘制圆形
        draw.ellipse(
            [(x - r, y - r), (x + r, y + r)],
            outline=click_color[:3],
            width=3
        )

        # 绘制十字
        draw.line([(x - cross_size, y), (x + cross_size, y)], fill=click_color[:3], width=2)
        draw.line([(x, y - cross_size), (x, y + cross_size)], fill=click_color[:3], width=2)

        # 绘制中心点
        draw.ellipse(
            [(x - 3, y - 3), (x + 3, y + 3)],
            fill=click_color[:3]
        )

        return self._image_to_base64(image)

    def draw_action_sequence(
        self,
        image_base64: str,
        actions: List[ActionMarker]
    ) -> str:
        """
        在截图上绘制操作序列轨迹

        Args:
            image_base64: 原始截图的 base64 字符串
            actions: 操作标记列表

        Returns:
            带标记的截图 base64 字符串
        """
        if not HAS_PIL or not actions:
            return image_base64

        image = self._base64_to_image(image_base64)
        if image is None:
            return image_base64

        draw = ImageDraw.Draw(image, 'RGBA')

        # 绘制连接线
        if len(actions) > 1:
            points = [action.point for action in actions]
            for i in range(len(points) - 1):
                draw.line(
                    [points[i], points[i + 1]],
                    fill=(100, 100, 100, 128),
                    width=2
                )

        # 绘制每个操作点
        for i, action in enumerate(actions):
            x, y = action.point

            # 根据操作类型选择颜色
            if action.action_type == 'click':
                color = self.style.click_color
            elif action.action_type == 'input':
                color = "#FFA500"  # 橙色
            elif action.action_type == 'scroll':
                color = "#0066FF"  # 蓝色
            else:
                color = "#888888"  # 灰色

            parsed_color = self._parse_color(color)

            # 绘制操作点
            r = 8
            draw.ellipse(
                [(x - r, y - r), (x + r, y + r)],
                fill=parsed_color[:3],
                outline=(255, 255, 255),
                width=2
            )

            # 绘制序号
            index = action.index if action.index is not None else i + 1
            font = self._get_font(10)
            index_text = str(index)

            # 序号背景圆
            idx_r = self.style.index_radius
            idx_x = x + r + 5
            idx_y = y - r - 5

            idx_bg_color = self._parse_color(self.style.index_bg_color)
            draw.ellipse(
                [(idx_x - idx_r, idx_y - idx_r), (idx_x + idx_r, idx_y + idx_r)],
                fill=idx_bg_color[:3]
            )

            # 序号文字
            text_bbox = draw.textbbox((0, 0), index_text, font=font)
            text_w = text_bbox[2] - text_bbox[0]
            text_h = text_bbox[3] - text_bbox[1]

            idx_color = self._parse_color(self.style.index_color)
            draw.text(
                (idx_x - text_w // 2, idx_y - text_h // 2),
                index_text,
                fill=idx_color[:3],
                font=font
            )

        return self._image_to_base64(image)

    def draw_element_with_click(
        self,
        image_base64: str,
        bbox: Tuple[int, int, int, int],
        click_point: Tuple[int, int],
        label: Optional[str] = None
    ) -> str:
        """
        同时绘制元素边界框和点击位置

        Args:
            image_base64: 原始截图的 base64 字符串
            bbox: 边界框坐标
            click_point: 点击位置坐标
            label: 标签文字

        Returns:
            带标记的截图 base64 字符串
        """
        # 先绘制边界框
        result = self.draw_bbox(image_base64, bbox, label=label)
        # 再绘制点击位置
        result = self.draw_click_point(result, click_point)
        return result

    def draw_multiple_elements(
        self,
        image_base64: str,
        elements: List[dict]
    ) -> str:
        """
        绘制多个元素标记

        Args:
            image_base64: 原始截图的 base64 字符串
            elements: 元素列表，每个元素包含 bbox, label, index 等

        Returns:
            带标记的截图 base64 字符串
        """
        if not HAS_PIL or not elements:
            return image_base64

        image = self._base64_to_image(image_base64)
        if image is None:
            return image_base64

        draw = ImageDraw.Draw(image, 'RGBA')

        # 颜色列表用于区分不同元素
        colors = [
            "#FF0000", "#00FF00", "#0066FF", "#FFA500",
            "#FF00FF", "#00FFFF", "#FFFF00", "#FF6666"
        ]

        for i, element in enumerate(elements):
            bbox = element.get('bbox')
            if not bbox:
                continue

            color = colors[i % len(colors)]
            label = element.get('label', f"Element {i + 1}")

            # 绘制边界框
            x1, y1, x2, y2 = bbox
            parsed_color = self._parse_color(color)

            draw.rectangle(
                [(x1, y1), (x2, y2)],
                outline=parsed_color[:3],
                width=self.style.bbox_width
            )

            # 绘制序号标记
            font = self._get_font(12)
            index_text = str(i + 1)

            idx_r = self.style.index_radius
            idx_x = x1 + idx_r
            idx_y = y1 + idx_r

            draw.ellipse(
                [(idx_x - idx_r, idx_y - idx_r), (idx_x + idx_r, idx_y + idx_r)],
                fill=parsed_color[:3]
            )

            text_bbox = draw.textbbox((0, 0), index_text, font=font)
            text_w = text_bbox[2] - text_bbox[0]
            text_h = text_bbox[3] - text_bbox[1]

            idx_color = self._parse_color(self.style.index_color)
            draw.text(
                (idx_x - text_w // 2, idx_y - text_h // 2),
                index_text,
                fill=idx_color[:3],
                font=font
            )

        return self._image_to_base64(image)


# 默认实例
_default_marker: Optional[ElementMarker] = None


def get_default_marker() -> ElementMarker:
    """获取默认的元素标记器实例"""
    global _default_marker
    if _default_marker is None:
        _default_marker = ElementMarker()
    return _default_marker


__all__ = [
    "ElementMarker",
    "MarkerStyle",
    "ActionMarker",
    "get_default_marker",
]



"""
JS 兼容的 HTML 报告生成器 - 完全复制 JS 版本的 React 可视化界面

此模块生成与 JS 版本 Midscene 完全兼容的 HTML 报告:
1. 使用相同的数据格式 (GroupedActionDump)
2. 嵌入 JS 版本的 React 可视化组件
3. 支持相同的交互功能（截图放大、步骤展开等）

报告文件命名格式: {driver}-{YYYY-MM-DD_HH-MM-SS}-{uuid8}.html
"""

import json
import uuid
import base64
import time
from datetime import datetime
from typing import List, Optional, Dict, Any, Tuple
from dataclasses import dataclass, field, asdict
from pathlib import Path

from ..shared.logger import logger


@dataclass
class MatchedElement:
    """匹配的元素信息 - 与 JS 版本 LocateResultElement 对齐"""
    id: str = ""
    reason: str = ""
    text: str = ""
    indexId: Optional[int] = None
    rect: Optional[Dict[str, int]] = None  # {left, top, width, height}
    center: Optional[List[int]] = None  # [x, y]

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "reason": self.reason,
            "text": self.text,
            "indexId": self.indexId,
            "rect": self.rect,
            "center": self.center,
        }


@dataclass
class ScreenshotInfo:
    """截图信息"""
    screenshot: str  # base64
    timing: str = "before"  # "before" | "after"

    def to_dict(self) -> dict:
        return {
            "type": "screenshot",
            "ts": int(time.time() * 1000),
            "screenshot": self.screenshot,
            "timing": self.timing,
        }


@dataclass
class TaskTiming:
    """任务时间信息"""
    start: int = 0  # 毫秒时间戳
    end: int = 0
    cost: int = 0  # 耗时毫秒

    def to_dict(self) -> dict:
        return {
            "start": self.start,
            "end": self.end,
            "cost": self.cost,
        }


@dataclass
class AIUsage:
    """AI 使用信息"""
    prompt_tokens: int = 0
    completion_tokens: int = 0
    total_tokens: int = 0

    def to_dict(self) -> dict:
        return {
            "prompt_tokens": self.prompt_tokens,
            "completion_tokens": self.completion_tokens,
            "total_tokens": self.total_tokens,
        }


@dataclass
class ExecutionTask:
    """
    执行任务 - 与 JS 版本 ExecutionTask 完全对齐

    对应 JS 版本的关键字段:
    - type: 任务类型 (Locate, Planning, Insight, Action)
    - subType: 子类型
    - status: 状态 (pending, running, finished, failed)
    - param: 任务参数
    - output: 输出结果
    - recorder: 截图记录数组
    - timing: 时间信息
    - usage: AI token 使用
    """
    type: str  # "Locate" | "Planning" | "Insight" | "Action"
    subType: Optional[str] = None
    subTask: bool = False
    status: str = "finished"  # "pending" | "running" | "finished" | "failed"

    # 参数 - 根据类型不同结构不同
    param: Optional[Dict[str, Any]] = None
    thought: Optional[str] = None

    # 输出
    output: Optional[Any] = None
    log: Optional[str] = None

    # 截图记录
    recorder: List[Dict] = field(default_factory=list)

    # 时间和资源
    timing: Optional[TaskTiming] = None
    usage: Optional[AIUsage] = None

    # 定位结果
    matchedElement: Optional[List[MatchedElement]] = None

    # 错误信息
    error: Optional[str] = None
    errorStack: Optional[str] = None

    # 缓存命中
    cacheHit: bool = False
    cacheType: Optional[str] = None

    def to_dict(self) -> dict:
        result = {
            "type": self.type,
            "subType": self.subType,
            "subTask": self.subTask,
            "status": self.status,
            "param": self.param,
            "thought": self.thought,
            "output": self.output,
            "log": self.log,
            "recorder": self.recorder,
        }

        if self.timing:
            result["timing"] = self.timing.to_dict()
        if self.usage:
            result["usage"] = self.usage.to_dict()
        if self.matchedElement:
            result["matchedElement"] = [e.to_dict() for e in self.matchedElement]
        if self.error:
            result["error"] = self.error
            result["errorStack"] = self.errorStack

        return result


@dataclass
class ExecutionDump:
    """
    执行记录 - 与 JS 版本 ExecutionDump 对齐

    对应一次完整的自动化执行会话
    """
    name: str
    description: Optional[str] = None
    tasks: List[ExecutionTask] = field(default_factory=list)
    logTime: int = 0  # 毫秒时间戳

    def to_dict(self) -> dict:
        return {
            "name": self.name,
            "description": self.description,
            "tasks": [t.to_dict() for t in self.tasks],
            "logTime": self.logTime or int(time.time() * 1000),
        }


@dataclass
class GroupedActionDump:
    """
    分组执行记录 - 与 JS 版本 GroupedActionDump 完全对齐

    这是 HTML 报告中嵌入的顶层数据结构
    """
    sdkVersion: str = "1.0.0-python"
    logTime: int = 0
    groupName: str = ""
    groupDescription: Optional[str] = None
    executions: List[ExecutionDump] = field(default_factory=list)
    modelBriefs: List[str] = field(default_factory=list)

    def to_dict(self) -> dict:
        return {
            "sdkVersion": self.sdkVersion,
            "logTime": self.logTime or int(time.time() * 1000),
            "groupName": self.groupName,
            "groupDescription": self.groupDescription,
            "executions": [e.to_dict() for e in self.executions],
            "modelBriefs": self.modelBriefs,
        }


class JSCompatibleReportGenerator:
    """
    JS 兼容的报告生成器

    生成与 JS 版本 Midscene 完全兼容的 HTML 报告。
    报告使用 JS 版本的 React 可视化组件，数据格式完全对齐。
    """

    VERSION = "1.0.0"

    # JS 版本报告的 HTML 模板 - 从 @midscene/core 提取
    # 包含完整的 React 应用代码，用于渲染与 JS 版本完全一致的 UI
    _JS_TEMPLATE_CACHE: Optional[str] = None
    
    @classmethod
    def _get_js_template(cls) -> Optional[str]:
        """
        获取 JS 版本的 React HTML 模板
        
        优先从本地缓存的 JS 模板文件加载，如果不存在则使用简化版模板
        """
        if cls._JS_TEMPLATE_CACHE is not None:
            return cls._JS_TEMPLATE_CACHE
        
        # 尝试从 JS 版本的 node_modules 中加载模板
        import os
        possible_paths = [
            # 常见的 JS 项目路径
            r"E:\code\qinghu\js-rpa-script\node_modules\@midscene\core\dist\lib\utils.js",
        ]
        
        for path in possible_paths:
            if os.path.exists(path):
                try:
                    with open(path, 'r', encoding='utf-8') as f:
                        content = f.read()
                    # 提取模板内容
                    import re
                    match = re.search(r'getReportTpl\(\)\s*\{\s*(?:const\s+reportTpl\s*=\s*)?/\*REPORT_HTML_REPLACED\*/\s*"([^"]*(?:\\.[^"]*)*)"', content)
                    if match:
                        template = match.group(1)
                        # 解码转义字符
                        template = template.encode().decode('unicode_escape')
                        cls._JS_TEMPLATE_CACHE = template
                        return template
                except Exception as e:
                    pass
        
        # 回退到简化版模板
        return None

    # JS 版本报告的 HTML 模板头部（简化版，当无法加载 JS 模板时使用）
    HTML_TEMPLATE_HEAD = '''<!doctype html>
<html>
  <head>
    <title>Report - PyMidscene</title>
    <link
      rel="icon"
      type="image/png"
      sizes="32x32"
      href="https://lf3-static.bytednsdoc.com/obj/eden-cn/vhaeh7vhabf/favicon-32x32.png"
    />
  <meta charset="UTF-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"></head>
  <body>
    <div id="root" style="height: 100vh; width: 100vw"></div>
'''

    # 简化版可视化界面（不依赖 React）
    VISUALIZER_TEMPLATE = '''
    <style>
      * { margin: 0; padding: 0; box-sizing: border-box; }
      body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif; background: #f5f7fa; }
      .container { max-width: 1400px; margin: 0 auto; padding: 20px; }

      /* Header */
      .header {
        background: linear-gradient(135deg, #667eea 0%, #764ba2 100%);
        color: white; padding: 24px; border-radius: 12px; margin-bottom: 20px;
        box-shadow: 0 4px 20px rgba(102, 126, 234, 0.3);
      }
      .header h1 { font-size: 24px; margin-bottom: 8px; display: flex; align-items: center; gap: 12px; }
      .header .logo { width: 36px; height: 36px; background: white; border-radius: 8px;
        display: flex; align-items: center; justify-content: center; font-weight: bold; color: #667eea; }
      .meta { display: flex; gap: 24px; font-size: 14px; opacity: 0.9; flex-wrap: wrap; }
      .meta-item { display: flex; gap: 6px; }
      .meta-item .label { opacity: 0.7; }

      /* Stats */
      .stats { display: grid; grid-template-columns: repeat(auto-fit, minmax(150px, 1fr)); gap: 16px; margin-bottom: 20px; }
      .stat-card { background: white; padding: 20px; border-radius: 10px; text-align: center;
        box-shadow: 0 2px 8px rgba(0,0,0,0.06); }
      .stat-card .number { font-size: 32px; font-weight: bold; color: #667eea; }
      .stat-card.success .number { color: #10b981; }
      .stat-card.failed .number { color: #ef4444; }
      .stat-card .label { font-size: 13px; color: #666; margin-top: 4px; }

      /* Timeline */
      .timeline { background: white; border-radius: 12px; box-shadow: 0 2px 8px rgba(0,0,0,0.06); overflow: hidden; }
      .timeline-header { padding: 16px 20px; border-bottom: 1px solid #eee; font-weight: 600; font-size: 16px;
        display: flex; justify-content: space-between; align-items: center; }
      .expand-all { font-size: 13px; color: #667eea; cursor: pointer; font-weight: normal; }

      /* Task */
      .task { border-bottom: 1px solid #f0f0f0; }
      .task:last-child { border-bottom: none; }
      .task-header { padding: 16px 20px; display: flex; align-items: center; gap: 16px; cursor: pointer;
        transition: background 0.15s; }
      .task-header:hover { background: #fafafa; }
      .task-index { width: 28px; height: 28px; border-radius: 50%; background: #667eea; color: white;
        display: flex; align-items: center; justify-content: center; font-size: 12px; font-weight: 600; flex-shrink: 0; }
      .task.success .task-index { background: #10b981; }
      .task.failed .task-index { background: #ef4444; }
      .task-info { flex: 1; min-width: 0; }
      .task-type { display: inline-flex; align-items: center; gap: 8px; }
      .type-badge { padding: 2px 8px; border-radius: 4px; font-size: 11px; font-weight: 600; text-transform: uppercase; }
      .type-badge.Locate { background: #fce7f3; color: #be185d; }
      .type-badge.Planning { background: #dbeafe; color: #1d4ed8; }
      .type-badge.Action { background: #d1fae5; color: #047857; }
      .type-badge.Insight { background: #fef3c7; color: #b45309; }
      .task-prompt { font-size: 14px; color: #333; margin-top: 4px;
        white-space: nowrap; overflow: hidden; text-overflow: ellipsis; max-width: 600px; }
      .task-meta { display: flex; gap: 16px; font-size: 12px; color: #999; }
      .task-toggle { width: 20px; color: #999; transition: transform 0.2s; }
      .task.expanded .task-toggle { transform: rotate(180deg); }

      /* Task Details */
      .task-details { display: none; padding: 0 20px 20px; background: #fafafa; }
      .task.expanded .task-details { display: block; }

      /* Screenshots */
      .screenshots { display: grid; grid-template-columns: repeat(auto-fit, minmax(400px, 1fr)); gap: 16px; margin-bottom: 16px; }
      .screenshot-panel { background: white; border-radius: 8px; overflow: hidden; box-shadow: 0 1px 4px rgba(0,0,0,0.08); }
      .screenshot-label { padding: 10px 12px; background: #f5f5f5; font-size: 12px; font-weight: 600; color: #666;
        border-bottom: 1px solid #eee; }
      .screenshot-img { width: 100%; display: block; cursor: zoom-in; }

      /* Details Grid */
      .details-grid { display: grid; grid-template-columns: repeat(auto-fit, minmax(280px, 1fr)); gap: 16px; }
      .detail-card { background: white; border-radius: 8px; padding: 16px; box-shadow: 0 1px 4px rgba(0,0,0,0.08); }
      .detail-card h4 { font-size: 12px; color: #666; margin-bottom: 12px; text-transform: uppercase; letter-spacing: 0.5px; }
      .detail-card pre { background: #f5f5f5; padding: 12px; border-radius: 6px; font-size: 12px;
        overflow-x: auto; white-space: pre-wrap; word-break: break-all; max-height: 200px; }
      .element-info .row { display: flex; justify-content: space-between; padding: 8px 0; border-bottom: 1px solid #f0f0f0; }
      .element-info .row:last-child { border-bottom: none; }
      .element-info .label { color: #666; font-size: 13px; }
      .element-info .value { font-weight: 500; color: #333; font-size: 13px; }

      /* Error */
      .error-box { background: #fef2f2; border: 1px solid #fecaca; color: #dc2626; padding: 12px;
        border-radius: 6px; font-size: 13px; margin-top: 16px; }

      /* Cache Hit Badge */
      .cache-badge { display: inline-flex; align-items: center; gap: 4px; padding: 2px 6px;
        background: #d1fae5; color: #047857; border-radius: 4px; font-size: 10px; font-weight: 600; }

      /* Modal */
      .modal { display: none; position: fixed; top: 0; left: 0; width: 100%; height: 100%;
        background: rgba(0,0,0,0.9); z-index: 1000; align-items: center; justify-content: center; }
      .modal.active { display: flex; }
      .modal img { max-width: 95%; max-height: 95%; object-fit: contain; }
      .modal-close { position: fixed; top: 20px; right: 20px; width: 40px; height: 40px; background: white;
        border-radius: 50%; display: flex; align-items: center; justify-content: center; cursor: pointer;
        font-size: 24px; color: #333; }

      /* Footer */
      .footer { text-align: center; padding: 24px; color: #999; font-size: 13px; }
      .footer a { color: #667eea; text-decoration: none; }
    </style>

    <div class="container">
      <div class="header">
        <h1><div class="logo">M</div>PyMidscene Execution Report</h1>
        <div class="meta">
          <div class="meta-item"><span class="label">Driver:</span><span id="driver-type"></span></div>
          <div class="meta-item"><span class="label">Started:</span><span id="start-time"></span></div>
          <div class="meta-item"><span class="label">Duration:</span><span id="duration"></span></div>
          <div class="meta-item"><span class="label">Status:</span><span id="status"></span></div>
        </div>
      </div>

      <div class="stats">
        <div class="stat-card"><div class="number" id="total-tasks">0</div><div class="label">Total Steps</div></div>
        <div class="stat-card success"><div class="number" id="success-tasks">0</div><div class="label">Successful</div></div>
        <div class="stat-card failed"><div class="number" id="failed-tasks">0</div><div class="label">Failed</div></div>
      </div>

      <div class="timeline">
        <div class="timeline-header">
          <span>Execution Steps</span>
          <span class="expand-all" onclick="toggleAllTasks()">Expand All</span>
        </div>
        <div id="tasks-container"></div>
      </div>

      <div class="footer">
        Generated by <a href="https://github.com/anthropics/pymidscene" target="_blank">PyMidscene</a> v{version}
        <br>{generation_time}
      </div>
    </div>

    <div class="modal" id="imageModal">
      <div class="modal-close" onclick="closeModal()">&times;</div>
      <img id="modalImage" src="" alt="Screenshot">
    </div>

    <script>
      // Report data embedded by Python
      const reportData = {report_data};

      // Initialize report
      function initReport() {
        const data = reportData;
        const execution = data.executions && data.executions[0];
        if (!execution) return;

        // Header info
        document.getElementById('driver-type').textContent = data.groupName || 'Playwright';
        document.getElementById('start-time').textContent = new Date(data.logTime).toLocaleString();

        const tasks = execution.tasks || [];
        const successCount = tasks.filter(t => t.status === 'finished').length;
        const failedCount = tasks.filter(t => t.status === 'failed').length;

        document.getElementById('total-tasks').textContent = tasks.length;
        document.getElementById('success-tasks').textContent = successCount;
        document.getElementById('failed-tasks').textContent = failedCount;
        document.getElementById('status').textContent = failedCount > 0 ? 'FAILED' : 'SUCCESS';

        // Calculate duration
        if (tasks.length > 0) {
          const totalDuration = tasks.reduce((sum, t) => sum + (t.timing?.cost || 0), 0);
          document.getElementById('duration').textContent = formatDuration(totalDuration);
        }

        // Render tasks
        const container = document.getElementById('tasks-container');
        tasks.forEach((task, index) => {
          container.appendChild(createTaskElement(task, index + 1));
        });
      }

      function formatDuration(ms) {
        if (ms < 1000) return ms + 'ms';
        const seconds = Math.floor(ms / 1000);
        if (seconds < 60) return seconds + 's';
        const minutes = Math.floor(seconds / 60);
        return minutes + 'm ' + (seconds % 60) + 's';
      }

      function createTaskElement(task, index) {
        const div = document.createElement('div');
        div.className = 'task ' + (task.status === 'finished' ? 'success' : task.status === 'failed' ? 'failed' : '');

        const prompt = getTaskPrompt(task);
        const duration = task.timing?.cost || 0;

        // Screenshots
        let screenshotsHtml = '';
        if (task.recorder && task.recorder.length > 0) {
          screenshotsHtml = '<div class="screenshots">';
          task.recorder.forEach(rec => {
            if (rec.screenshot) {
              const imgSrc = rec.screenshot.startsWith('data:') ? rec.screenshot : 'data:image/png;base64,' + rec.screenshot;
              screenshotsHtml += `
                <div class="screenshot-panel">
                  <div class="screenshot-label">${rec.timing === 'before' ? 'Before' : 'After'}</div>
                  <img class="screenshot-img" src="${imgSrc}" onclick="openModal(this.src)">
                </div>`;
            }
          });
          screenshotsHtml += '</div>';
        }

        // Element info
        let elementHtml = '';
        if (task.matchedElement && task.matchedElement.length > 0) {
          const elem = task.matchedElement[0];
          elementHtml = `
            <div class="detail-card">
              <h4>Element Location</h4>
              <div class="element-info">
                ${elem.rect ? `<div class="row"><span class="label">Bounding Box</span><span class="value">[${elem.rect.left}, ${elem.rect.top}, ${elem.rect.width}, ${elem.rect.height}]</span></div>` : ''}
                ${elem.center ? `<div class="row"><span class="label">Click Point</span><span class="value">(${elem.center[0]}, ${elem.center[1]})</span></div>` : ''}
                ${elem.text ? `<div class="row"><span class="label">Text</span><span class="value">${elem.text}</span></div>` : ''}
              </div>
            </div>`;
        }

        // AI info
        let aiHtml = '';
        if (task.usage) {
          aiHtml = `
            <div class="detail-card">
              <h4>AI Information</h4>
              <div class="element-info">
                <div class="row"><span class="label">Total Tokens</span><span class="value">${task.usage.total_tokens || 0}</span></div>
                <div class="row"><span class="label">Prompt Tokens</span><span class="value">${task.usage.prompt_tokens || 0}</span></div>
                <div class="row"><span class="label">Completion Tokens</span><span class="value">${task.usage.completion_tokens || 0}</span></div>
              </div>
            </div>`;
        }

        // Error
        let errorHtml = '';
        if (task.error) {
          errorHtml = `<div class="error-box">${task.error}</div>`;
        }

        div.innerHTML = `
          <div class="task-header" onclick="toggleTask(this)">
            <div class="task-index">${index}</div>
            <div class="task-info">
              <div class="task-type">
                <span class="type-badge ${task.type}">${task.type}</span>
                ${task.cacheHit ? '<span class="cache-badge">CACHE HIT</span>' : ''}
              </div>
              <div class="task-prompt">${escapeHtml(prompt)}</div>
            </div>
            <div class="task-meta">
              <span>${formatDuration(duration)}</span>
            </div>
            <div class="task-toggle">▼</div>
          </div>
          <div class="task-details">
            ${screenshotsHtml}
            <div class="details-grid">
              ${elementHtml}
              ${aiHtml}
            </div>
            ${errorHtml}
          </div>`;

        return div;
      }

      function getTaskPrompt(task) {
        if (!task.param) return task.type;
        if (typeof task.param === 'string') return task.param;
        if (task.param.prompt) return task.param.prompt;
        if (task.param.value) return task.param.value;
        if (task.param.locate?.prompt) return task.param.locate.prompt;
        return JSON.stringify(task.param).slice(0, 100);
      }

      function escapeHtml(text) {
        const div = document.createElement('div');
        div.textContent = text;
        return div.innerHTML;
      }

      function toggleTask(header) {
        header.parentElement.classList.toggle('expanded');
      }

      function toggleAllTasks() {
        const tasks = document.querySelectorAll('.task');
        const allExpanded = Array.from(tasks).every(t => t.classList.contains('expanded'));
        tasks.forEach(t => {
          if (allExpanded) t.classList.remove('expanded');
          else t.classList.add('expanded');
        });
      }

      function openModal(src) {
        document.getElementById('modalImage').src = src;
        document.getElementById('imageModal').classList.add('active');
      }

      function closeModal() {
        document.getElementById('imageModal').classList.remove('active');
      }

      document.getElementById('imageModal').addEventListener('click', function(e) {
        if (e.target.id === 'imageModal') closeModal();
      });

      document.addEventListener('keydown', function(e) {
        if (e.key === 'Escape') closeModal();
      });

      // Initialize on load
      initReport();
    </script>
'''

    HTML_TEMPLATE_TAIL = '''
  </body>
</html>'''

    def __init__(self):
        """初始化报告生成器"""
        self._current_dump: Optional[GroupedActionDump] = None
        self._current_execution: Optional[ExecutionDump] = None

    def start_session(
        self,
        group_name: str = "PyMidscene Execution",
        description: Optional[str] = None,
        model_name: Optional[str] = None
    ):
        """
        开始新的报告会话

        Args:
            group_name: 组名称
            description: 描述
            model_name: 使用的模型名称
        """
        self._current_dump = GroupedActionDump(
            sdkVersion=f"{self.VERSION}-python",
            logTime=int(time.time() * 1000),
            groupName=group_name,
            groupDescription=description,
            executions=[],
            modelBriefs=[model_name] if model_name else [],
        )

        self._current_execution = ExecutionDump(
            name=group_name,
            description=description,
            tasks=[],
            logTime=int(time.time() * 1000),
        )

        self._current_dump.executions.append(self._current_execution)

    def add_task(
        self,
        task_type: str,
        prompt: str,
        status: str = "finished",
        screenshot_before: Optional[str] = None,
        screenshot_after: Optional[str] = None,
        screenshot_marked: Optional[str] = None,
        element_rect: Optional[Dict[str, int]] = None,
        element_center: Optional[List[int]] = None,
        element_text: Optional[str] = None,
        duration_ms: int = 0,
        ai_tokens: Optional[int] = None,
        ai_prompt_tokens: Optional[int] = None,
        ai_completion_tokens: Optional[int] = None,
        error: Optional[str] = None,
        cache_hit: bool = False,
        thought: Optional[str] = None,
    ):
        """
        添加执行任务

        Args:
            task_type: 任务类型 (Locate, Planning, Action, Insight)
            prompt: 用户指令
            status: 状态 (finished, failed, pending)
            screenshot_before: 操作前截图 (base64)
            screenshot_after: 操作后截图 (base64)
            screenshot_marked: 带标记的截图 (base64)
            element_rect: 元素边界框 {left, top, width, height}
            element_center: 元素中心点 [x, y]
            element_text: 元素文本
            duration_ms: 耗时毫秒
            ai_tokens: AI token 总数
            ai_prompt_tokens: 提示 token 数
            ai_completion_tokens: 完成 token 数
            error: 错误信息
            cache_hit: 是否缓存命中
            thought: AI 思考过程
        """
        if not self._current_execution:
            self.start_session()

        # 构建截图记录
        recorder = []
        if screenshot_before:
            recorder.append({
                "type": "screenshot",
                "ts": int(time.time() * 1000),
                "screenshot": screenshot_before,
                "timing": "before",
            })
        if screenshot_marked:
            recorder.append({
                "type": "screenshot",
                "ts": int(time.time() * 1000),
                "screenshot": screenshot_marked,
                "timing": "after",
            })
        elif screenshot_after:
            recorder.append({
                "type": "screenshot",
                "ts": int(time.time() * 1000),
                "screenshot": screenshot_after,
                "timing": "after",
            })

        # 构建匹配元素
        matched_element = None
        if element_rect or element_center:
            matched_element = [MatchedElement(
                id=str(uuid.uuid4())[:8],
                reason=prompt,
                text=element_text or "",
                rect=element_rect,
                center=element_center,
            )]

        # 构建 AI 使用信息
        usage = None
        if ai_tokens or ai_prompt_tokens or ai_completion_tokens:
            usage = AIUsage(
                prompt_tokens=ai_prompt_tokens or 0,
                completion_tokens=ai_completion_tokens or 0,
                total_tokens=ai_tokens or (ai_prompt_tokens or 0) + (ai_completion_tokens or 0),
            )

        # 构建时间信息
        timing = TaskTiming(
            start=int(time.time() * 1000) - duration_ms,
            end=int(time.time() * 1000),
            cost=duration_ms,
        )

        task = ExecutionTask(
            type=task_type,
            status=status,
            param={"prompt": prompt},
            thought=thought,
            recorder=recorder,
            matchedElement=matched_element,
            timing=timing,
            usage=usage,
            error=error,
            cacheHit=cache_hit,
        )

        if self._current_execution is None:
            self.start_session()
        # 类型断言：此时 _current_execution 一定不为 None
        assert self._current_execution is not None
        self._current_execution.tasks.append(task)

    def generate_html(self) -> str:
        """
        生成 HTML 报告

        Returns:
            完整的 HTML 字符串
        """
        if not self._current_dump:
            self.start_session()
        
        # 类型断言：此时 _current_dump 一定不为 None
        assert self._current_dump is not None
        report_data = json.dumps(
            self._current_dump.to_dict(),
            ensure_ascii=False,
            separators=(',', ':')
        )

        html = self.HTML_TEMPLATE_HEAD
        html += self.VISUALIZER_TEMPLATE.format(
            version=self.VERSION,
            generation_time=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            report_data=report_data,
        )
        html += self.HTML_TEMPLATE_TAIL

        return html

    def save(
        self,
        report_dir: str,
        driver_type: str = "playwright",
        filename: Optional[str] = None
    ) -> str:
        """
        保存 HTML 报告到文件

        Args:
            report_dir: 报告目录
            driver_type: 驱动类型
            filename: 文件名（可选，默认自动生成）

        Returns:
            保存的文件路径
        """
        # 生成文件名
        if filename is None:
            timestamp = datetime.now()
            session_id = uuid.uuid4().hex[:8]
            filename = f"{driver_type}-{timestamp.strftime('%Y-%m-%d_%H-%M-%S')}-{session_id}.html"

        # 确保目录存在
        report_path = Path(report_dir)
        report_path.mkdir(parents=True, exist_ok=True)

        # 生成并保存 HTML
        html_content = self.generate_html()
        file_path = report_path / filename

        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(html_content)

        logger.info(f"Report saved to: {file_path}")
        return str(file_path)

    def reset(self):
        """重置报告生成器"""
        self._current_dump = None
        self._current_execution = None


# 全局实例
_default_js_generator: Optional[JSCompatibleReportGenerator] = None


def get_js_report_generator() -> JSCompatibleReportGenerator:
    """获取默认的 JS 兼容报告生成器"""
    global _default_js_generator
    if _default_js_generator is None:
        _default_js_generator = JSCompatibleReportGenerator()
    return _default_js_generator


__all__ = [
    "JSCompatibleReportGenerator",
    "GroupedActionDump",
    "ExecutionDump",
    "ExecutionTask",
    "MatchedElement",
    "AIUsage",
    "TaskTiming",
    "get_js_report_generator",
]



"""
AI 模型基类 - 对应 packages/core/src/ai-model/models/

定义 AI 模型的抽象接口。
"""

from abc import ABC, abstractmethod
from typing import Dict, Any, List, Optional
from ....shared.types import AIUsageInfo


class BaseAIModel(ABC):
    """AI 模型抽象基类"""

    def __init__(
        self,
        model_name: str,
        api_key: str,
        base_url: Optional[str] = None,
        **kwargs
    ):
        self.model_name = model_name
        self.api_key = api_key
        self.base_url = base_url
        self.config = kwargs

    @abstractmethod
    def call(
        self,
        messages: List[Dict[str, Any]],
        **kwargs
    ) -> Dict[str, Any]:
        """
        调用 AI 模型

        Args:
            messages: 消息列表
            **kwargs: 其他参数

        Returns:
            包含 content 和 usage 的字典
        """
        pass

    @abstractmethod
    def validate_config(self) -> bool:
        """验证配置是否有效"""
        pass


__all__ = ["BaseAIModel"]



"""
千问模型适配器 - Qwen VL 模型支持

提供对阿里云千问视觉语言模型的支持。
"""

from typing import Dict, Any, List, Optional
from .base import BaseAIModel
from ..service_caller import ModelConfig, call_ai


class QwenVLModel(BaseAIModel):
    """千问 VL 模型适配器"""

    # 默认配置
    DEFAULT_BASE_URL = "https://dashscope.aliyuncs.com/compatible-mode/v1"
    SUPPORTED_MODELS = [
        "qwen-vl-max",
        "qwen-vl-plus",
        "qwen2-vl-7b-instruct",
        "qwen2-vl-72b-instruct",
    ]

    def __init__(
        self,
        model_name: str = "qwen-vl-max",
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        temperature: float = 0.0,
        max_tokens: int = 4096,
        **kwargs
    ):
        """
        初始化千问模型

        Args:
            model_name: 模型名称（qwen-vl-max 或 qwen-vl-plus）
            api_key: API 密钥（千问的 API Key）
            base_url: API 基础 URL
            temperature: 温度参数
            max_tokens: 最大 token 数
            **kwargs: 其他参数
        """
        # 如果没有提供 base_url，使用默认值
        if base_url is None:
            base_url = self.DEFAULT_BASE_URL

        super().__init__(
            model_name=model_name,
            api_key=api_key or "",
            base_url=base_url,
            **kwargs
        )

        self.temperature = temperature
        self.max_tokens = max_tokens

        # 验证模型名称
        if model_name not in self.SUPPORTED_MODELS:
            raise ValueError(
                f"Unsupported model: {model_name}. "
                f"Supported models: {', '.join(self.SUPPORTED_MODELS)}"
            )

    def validate_config(self) -> bool:
        """验证配置是否有效"""
        if not self.api_key:
            raise ValueError("API key is required for Qwen VL model")

        if not self.base_url:
            raise ValueError("Base URL is required for Qwen VL model")

        return True

    def call(
        self,
        messages: List[Dict[str, Any]],
        **kwargs
    ) -> Dict[str, Any]:
        """
        调用千问模型

        Args:
            messages: 消息列表（OpenAI 格式）
            **kwargs: 其他参数（如 temperature, max_tokens）

        Returns:
            包含 content 和 usage 的字典
        """
        # 验证配置
        self.validate_config()

        # 创建模型配置
        model_config = ModelConfig(
            model_name=self.model_name,
            base_url=self.base_url,
            api_key=self.api_key,
            temperature=kwargs.get("temperature", self.temperature),
            max_tokens=kwargs.get("max_tokens", self.max_tokens),
            timeout=kwargs.get("timeout", 60000),
            retry_count=kwargs.get("retry_count", 1),
            retry_interval=kwargs.get("retry_interval", 2000),
            http_proxy=kwargs.get("http_proxy"),
            model_description=f"Qwen VL {self.model_name}",
            intent=kwargs.get("intent"),
        )

        # 调用 AI
        result = call_ai(
            messages=messages,
            model_config=model_config,
            stream=kwargs.get("stream", False),
            on_chunk=kwargs.get("on_chunk"),
        )

        return result

    @classmethod
    def from_env(cls, model_name: str = "qwen-vl-max") -> 'QwenVLModel':
        """
        从环境变量创建模型实例

        Args:
            model_name: 模型名称

        Returns:
            QwenVLModel 实例
        """
        import os

        api_key = os.getenv("MIDSCENE_QWEN_API_KEY")
        base_url = os.getenv(
            "MIDSCENE_QWEN_BASE_URL",
            cls.DEFAULT_BASE_URL
        )

        if not api_key:
            raise ValueError(
                "MIDSCENE_QWEN_API_KEY environment variable is required"
            )

        return cls(
            model_name=model_name,
            api_key=api_key,
            base_url=base_url,
        )


__all__ = ["QwenVLModel"]



"""
PlaywrightAgent - 对应 JS 版本的 PlaywrightAgent

简化 API，直接传入 Playwright page 对象即可使用。
与 JS 版本对齐，默认启用日志记录，自动生成 HTML 报告。

示例:
    from playwright.async_api import async_playwright
    from pymidscene import PlaywrightAgent

    # 方式1：通过环境变量配置（推荐）
    os.environ["MIDSCENE_MODEL_NAME"] = "qwen-vl-max"
    os.environ["MIDSCENE_MODEL_API_KEY"] = "your-key"
    os.environ["MIDSCENE_MODEL_BASE_URL"] = "https://dashscope.aliyuncs.com/compatible-mode/v1"
    os.environ["MIDSCENE_MODEL_FAMILY"] = "qwen2.5-vl"

    agent = PlaywrightAgent(page)
    await agent.ai_click("登录按钮")

    # 结束时生成报告
    agent.finish()

    # 方式2：使用上下文管理器（自动生成报告）
    async with PlaywrightAgent(page) as agent:
        await agent.ai_click("登录按钮")
    # 退出时自动调用 finish()
"""

from typing import Optional, Dict, Any, Union
from playwright.async_api import Page as AsyncPlaywrightPage

try:
    from playwright.sync_api import Page as SyncPlaywrightPage
except ImportError:
    SyncPlaywrightPage = None

from .page import WebPage
from ...core.agent.agent import Agent
from ...shared.logger import logger


class PlaywrightAgent:
    """
    Playwright Agent - 简化的 API 入口

    直接传入 Playwright page 对象，无需手动创建 WebPage 适配器。
    对应 JS 版本的 `new PlaywrightAgent(page, opts)`。

    与 JS 版本对齐：
    - 默认启用日志记录
    - 自动在当前目录创建 midscene_run 目录
    - 调用 finish() 或使用上下文管理器自动生成 HTML 报告

    Args:
        page: Playwright Page 实例（支持同步和异步）
        model_config: 模型配置字典，键为环境变量名称
            - MIDSCENE_MODEL_NAME: 模型名称（如 "qwen-vl-max"）
            - MIDSCENE_MODEL_API_KEY: API 密钥
            - MIDSCENE_MODEL_BASE_URL: API 基础 URL
            - MIDSCENE_MODEL_FAMILY: 模型家族（qwen2.5-vl, doubao-vision 等）
        cache_id: 缓存 ID，用于标识缓存文件
        cache_strategy: 缓存策略
            - "read-write": 读写缓存（默认）
            - "read-only": 只读缓存
            - "write-only": 只写缓存
        enable_recording: 是否启用执行记录（默认启用）
        report_dir: 报告保存目录（默认为当前目录）

    示例:
        # 基础用法
        agent = PlaywrightAgent(page)
        await agent.ai_click("登录按钮")
        agent.finish()  # 生成报告

        # 使用上下文管理器（推荐）
        async with PlaywrightAgent(page) as agent:
            await agent.ai_click("登录按钮")
        # 自动生成报告
    """

    def __init__(
        self,
        page: Union[AsyncPlaywrightPage, "SyncPlaywrightPage"],
        model_config: Optional[Dict[str, Any]] = None,
        cache_id: Optional[str] = None,
        cache_strategy: str = "read-write",
        enable_recording: bool = True,  # 默认启用，与 JS 版本对齐
        report_dir: Optional[str] = None,
        wait_for_navigation_timeout: Optional[int] = None,
        wait_for_network_idle_timeout: Optional[int] = None,
    ):
        """初始化 PlaywrightAgent"""
        # 创建 WebPage 适配器
        self._web_page = WebPage(
            page,
            wait_for_navigation_timeout=wait_for_navigation_timeout,
            wait_for_network_idle_timeout=wait_for_network_idle_timeout,
        )

        # 创建内部 Agent
        self._agent = Agent(
            interface=self._web_page,
            model_config=model_config,
            cache_id=cache_id,
            cache_strategy=cache_strategy,
            enable_recording=enable_recording,
            driver_type="playwright",
            report_dir=report_dir,
        )

        # 保存原始 page 引用
        self.page = page

        logger.info(
            f"PlaywrightAgent initialized: "
            f"cache_id={cache_id}, recording={enable_recording}"
        )

    # ==================== 核心 AI 方法 ====================

    async def ai_locate(self, description: str) -> Optional[Dict[str, Any]]:
        """
        AI 元素定位

        Args:
            description: 元素的自然语言描述

        Returns:
            定位结果，包含 center、rect 等信息
        """
        return await self._agent.ai_locate(description)

    async def ai_click(self, description: str) -> bool:
        """
        AI 点击操作

        Args:
            description: 要点击元素的自然语言描述

        Returns:
            是否成功点击
        """
        return await self._agent.ai_click(description)

    async def ai_input(self, description: str, text: str) -> bool:
        """
        AI 输入操作

        Args:
            description: 输入框的自然语言描述
            text: 要输入的文本

        Returns:
            是否成功输入
        """
        return await self._agent.ai_input(description, text)

    async def ai_query(
        self,
        data_schema: Union[str, Dict[str, str]],
        data_description: Optional[str] = None
    ) -> Dict[str, Any]:
        """
        AI 数据提取

        Args:
            data_schema: 数据结构定义
            data_description: 数据描述

        Returns:
            提取的结构化数据
        """
        return await self._agent.ai_query(data_schema, data_description)

    async def ai_assert(self, assertion: str, error_message: Optional[str] = None):
        """
        AI 断言

        Args:
            assertion: 断言条件描述
            error_message: 断言失败时的错误消息

        Raises:
            AssertionError: 断言失败时抛出
        """
        return await self._agent.ai_assert(assertion, error_message)

    async def ai_action(self, action: str) -> bool:
        """
        执行 AI 动作（别名方法，兼容 JS 版本的 aiAction）

        Args:
            action: 动作的自然语言描述

        Returns:
            是否成功执行
        """
        return await self._agent.ai_click(action)

    # ==================== 日志/报告方法 ====================

    def finish(self) -> Optional[str]:
        """
        结束会话并生成 HTML 报告

        与 JS 版本对齐，在任务完成后调用此方法生成可视化报告。
        报告将保存到 midscene_run/report/ 目录。

        Returns:
            报告文件路径（如果启用了记录）
        """
        return self._agent.finish()

    def save_report(self) -> Optional[str]:
        """
        手动保存报告（不结束会话）

        Returns:
            报告文件路径
        """
        return self._agent.save_report()

    def get_report_dir(self) -> Optional[str]:
        """
        获取报告目录路径

        Returns:
            报告目录的绝对路径
        """
        return self._agent.get_report_dir()

    # ==================== 辅助方法 ====================

    def get_cache_stats(self) -> Optional[Dict[str, Any]]:
        """获取缓存统计信息"""
        return self._agent.get_cache_stats()

    @property
    def recorder(self):
        """获取执行记录器"""
        return self._agent.recorder

    @property
    def session_recorder(self):
        """获取会话记录器"""
        return self._agent.session_recorder

    @property
    def interface(self):
        """获取底层 WebPage 接口"""
        return self._web_page

    @property
    def agent(self):
        """获取底层 Agent 实例"""
        return self._agent

    # ==================== 上下文管理器 ====================

    async def __aenter__(self):
        """异步上下文管理器入口"""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """异步上下文管理器退出 - 自动保存报告"""
        if exc_type and self._agent.session_recorder:
            # 如果有异常，标记当前步骤失败
            if self._agent.session_recorder.current_step:
                self._agent.session_recorder.fail_step(str(exc_val))

        self.finish()
        return False


__all__ = ["PlaywrightAgent"]



"""
Prompt 模板模块

提供 AI 模型的各种 Prompt 模板。
"""

from .locator import system_prompt_to_locate_element, find_element_prompt
from .extractor import system_prompt_to_extract, extract_data_prompt, parse_xml_extraction_response
from .planner import system_prompt_to_plan

__all__ = [
    # Locator prompts
    "system_prompt_to_locate_element",
    "find_element_prompt",
    # Extractor prompts
    "system_prompt_to_extract",
    "extract_data_prompt",
    "parse_xml_extraction_response",
    # Planner prompts
    "system_prompt_to_plan",
]



"""
核心类型定义 - 对应 packages/core/src/types.ts

这个模块定义了 PyMidscene 核心功能中使用的类型。
"""

from typing import TypedDict, Optional, List, Dict, Any, Callable, Tuple, Protocol, Literal
from dataclasses import dataclass, field
from abc import ABC, abstractmethod

from ..shared.types import (
    Size,
    Rect,
    LocateResultElement,
    AIUsageInfo,
    TaskStatus,
    ExecutionTaskType,
    ExecutionTaskTiming,
)


# ============================================================================
# UI 上下文
# ============================================================================

class ScreenshotItem:
    """截图项（简化版，完整实现在 screenshot.py）"""
    def __init__(self, data: str):
        self.data = data  # Base64 编码的图像数据

    def to_serializable(self) -> str:
        """转换为可序列化格式"""
        return self.data

    @staticmethod
    def is_serialized_data(value: Any) -> bool:
        """检查是否为序列化数据"""
        return isinstance(value, str)

    @staticmethod
    def from_serialized_data(data: str) -> 'ScreenshotItem':
        """从序列化数据恢复"""
        return ScreenshotItem(data)


@dataclass
class UIContext(ABC):
    """UI 上下文抽象基类"""
    screenshot: ScreenshotItem
    size: Size
    _is_frozen: bool = False


# ============================================================================
# Service 相关类型
# ============================================================================

@dataclass
class ServiceTaskInfo:
    """Service 任务信息"""
    duration_ms: float
    format_response: Optional[str] = None
    raw_response: Optional[str] = None
    usage: Optional[AIUsageInfo] = None
    search_area: Optional[Rect] = None
    search_area_raw_response: Optional[str] = None
    search_area_usage: Optional[AIUsageInfo] = None
    reasoning_content: Optional[str] = None


@dataclass
class ServiceDump:
    """Service 执行记录"""
    type: str  # 'locate' | 'extract' | 'assert'
    log_id: str
    log_time: float
    user_query: Dict[str, Any]
    matched_element: List[LocateResultElement]
    matched_rect: Optional[Rect]
    deep_think: Optional[bool]
    data: Any
    assertion_pass: Optional[bool]
    assertion_thought: Optional[str]
    task_info: ServiceTaskInfo
    error: Optional[str]
    output: Optional[Any]


class ServiceError(Exception):
    """Service 错误"""
    def __init__(self, message: str, dump: ServiceDump):
        super().__init__(message)
        self.dump = dump


# ============================================================================
# 执行任务相关类型
# ============================================================================

@dataclass
class ExecutionRecorderItem:
    """执行记录项"""
    type: str  # 'screenshot'
    ts: float  # 时间戳
    screenshot: Optional[ScreenshotItem] = None
    timing: Optional[str] = None


@dataclass
class ExecutionTaskHitBy:
    """任务命中来源"""
    from_: str  # 使用 from_ 避免与 Python 关键字冲突
    context: Dict[str, Any]


@dataclass
class ExecutorContext:
    """执行器上下文"""
    task: 'ExecutionTask'
    element: Optional[LocateResultElement] = None
    ui_context: Optional[UIContext] = None


# 执行器函数类型
ExecutorFunction = Callable[
    [Any, ExecutorContext],
    Optional[Dict[str, Any]]
]


@dataclass
class ExecutionTask:
    """执行任务"""
    type: ExecutionTaskType
    sub_type: Optional[str] = None
    sub_task: bool = False
    param: Optional[Any] = None
    thought: Optional[str] = None
    ui_context: Optional[UIContext] = None

    # 执行状态
    status: TaskStatus = "pending"
    error: Optional[Exception] = None
    error_message: Optional[str] = None
    error_stack: Optional[str] = None

    # 执行结果
    output: Optional[Any] = None
    log: Optional[Any] = None
    recorder: List[ExecutionRecorderItem] = field(default_factory=list)
    hit_by: Optional[ExecutionTaskHitBy] = None

    # 时间和资源统计
    timing: Optional[ExecutionTaskTiming] = None
    usage: Optional[AIUsageInfo] = None
    search_area_usage: Optional[AIUsageInfo] = None
    reasoning_content: Optional[str] = None


# ============================================================================
# 执行记录（Dump）
# ============================================================================

@dataclass
class DumpMeta:
    """Dump 元数据"""
    log_time: float


@dataclass
class ExecutionDump:
    """执行记录"""
    log_time: float
    name: str
    description: Optional[str]
    tasks: List[ExecutionTask]
    ai_act_context: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典（用于序列化）"""
        return {
            "logTime": self.log_time,
            "name": self.name,
            "description": self.description,
            "tasks": [self._task_to_dict(task) for task in self.tasks],
            "aiActContext": self.ai_act_context,
        }

    def _task_to_dict(self, task: ExecutionTask) -> Dict[str, Any]:
        """将任务转换为字典"""
        # 简化实现，实际需要处理所有字段
        return {
            "type": task.type,
            "status": task.status,
            "param": task.param,
            # ... 其他字段
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'ExecutionDump':
        """从字典创建实例"""
        return ExecutionDump(
            log_time=data["logTime"],
            name=data["name"],
            description=data.get("description"),
            tasks=[],  # 需要实现完整的反序列化
            ai_act_context=data.get("aiActContext"),
        )


@dataclass
class GroupedActionDump:
    """分组执行记录"""
    sdk_version: str
    group_name: str
    group_description: Optional[str]
    model_briefs: List[str]
    executions: List[ExecutionDump]

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典（用于序列化）"""
        return {
            "sdkVersion": self.sdk_version,
            "groupName": self.group_name,
            "groupDescription": self.group_description,
            "modelBriefs": self.model_briefs,
            "executions": [exec.to_dict() for exec in self.executions],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'GroupedActionDump':
        """从字典创建实例"""
        return GroupedActionDump(
            sdk_version=data["sdkVersion"],
            group_name=data["groupName"],
            group_description=data.get("groupDescription"),
            model_briefs=data["modelBriefs"],
            executions=[
                ExecutionDump.from_dict(exec) for exec in data["executions"]
            ],
        )


# ============================================================================
# Planning 相关类型
# ============================================================================

@dataclass
class PlanningAction:
    """规划动作"""
    type: str
    param: Any
    thought: Optional[str] = None
    log: Optional[str] = None


@dataclass
class PlanningAIResponse:
    """AI 规划响应"""
    actions: Optional[List[PlanningAction]] = None
    usage: Optional[AIUsageInfo] = None
    raw_response: Optional[str] = None
    yaml_string: Optional[str] = None
    error: Optional[str] = None
    reasoning_content: Optional[str] = None
    should_continue_planning: bool = False


# ============================================================================
# Agent 配置类型
# ============================================================================

ThinkingLevel = Literal["off", "medium", "high"]
DeepThinkOption = Literal["unset"] | bool


@dataclass
class AgentWaitForOpt:
    """Agent 等待选项"""
    check_interval_ms: int = 1000
    timeout_ms: int = 30000


@dataclass
class AgentAssertOpt:
    """Agent 断言选项"""
    keep_raw_response: bool = False


# ============================================================================
# 导出所有类型
# ============================================================================

__all__ = [
    # UI 上下文
    "ScreenshotItem",
    "UIContext",
    # Service 相关
    "ServiceTaskInfo",
    "ServiceDump",
    "ServiceError",
    # 执行任务
    "ExecutionRecorderItem",
    "ExecutionTaskHitBy",
    "ExecutorContext",
    "ExecutorFunction",
    "ExecutionTask",
    # 执行记录
    "DumpMeta",
    "ExecutionDump",
    "GroupedActionDump",
    # Planning
    "PlanningAction",
    "PlanningAIResponse",
    # Agent 配置
    "ThinkingLevel",
    "DeepThinkOption",
    "AgentWaitForOpt",
    "AgentAssertOpt",
]



"""
PyMidscene Core 模块

提供核心功能：
- 执行记录和报告生成
- 目录结构管理
- 元素标记绘制
- HTML 可视化报告
"""

from .types import (
    ScreenshotItem,
    UIContext,
    ServiceTaskInfo,
    ServiceDump,
    ServiceError,
    ExecutionRecorderItem,
    ExecutionTask,
    ExecutionDump,
    GroupedActionDump,
    PlanningAction,
    PlanningAIResponse,
)

from .dump import (
    ExecutionRecorder,
    SessionRecorder,
    GroupedExecutionRecorder,
    ServiceDumpBuilder,
    create_execution_recorder,
    create_session_recorder,
    create_grouped_recorder,
)

from .run_manager import (
    MidsceneRunManager,
    get_default_run_manager,
)

from .element_marker import (
    ElementMarker,
    MarkerStyle,
    ActionMarker,
    get_default_marker,
)

from .report_generator import (
    HTMLReportGenerator,
    ReportSession,
    ReportStep,
    get_default_report_generator,
)

from .js_report_generator import (
    JSCompatibleReportGenerator,
    GroupedActionDump,
    ExecutionDump as JSExecutionDump,
    ExecutionTask as JSExecutionTask,
    MatchedElement,
    AIUsage,
    TaskTiming,
    get_js_report_generator,
)

from .logging_system import (
    MidsceneLogManager,
    MidsceneFormatter,
    get_log_manager,
    reset_log_manager,
)


__all__ = [
    # Types
    "ScreenshotItem",
    "UIContext",
    "ServiceTaskInfo",
    "ServiceDump",
    "ServiceError",
    "ExecutionRecorderItem",
    "ExecutionTask",
    "ExecutionDump",
    "GroupedActionDump",
    "PlanningAction",
    "PlanningAIResponse",
    # Dump / Recording
    "ExecutionRecorder",
    "SessionRecorder",
    "GroupedExecutionRecorder",
    "ServiceDumpBuilder",
    "create_execution_recorder",
    "create_session_recorder",
    "create_grouped_recorder",
    # Run Manager
    "MidsceneRunManager",
    "get_default_run_manager",
    # Element Marker
    "ElementMarker",
    "MarkerStyle",
    "ActionMarker",
    "get_default_marker",
    # Report Generator
    "HTMLReportGenerator",
    "ReportSession",
    "ReportStep",
    "get_default_report_generator",
]



"""
Playwright 页面适配器 - 对应 packages/web-integration/src/playwright/page.ts

将 Playwright Page 适配为 PyMidscene 的统一接口。
"""

from typing import Optional, Any
import base64
import asyncio

from playwright.async_api import Page as PlaywrightPage

from ..base import AbstractInterface
from ...core.types import UIContext, ScreenshotItem
from ...shared.types import Size
from ...shared.logger import logger


class WebPage(AbstractInterface):
    """Playwright 页面适配器"""

    # 默认配置
    DEFAULT_WAIT_FOR_NAVIGATION_TIMEOUT = 10000  # 10 秒
    DEFAULT_WAIT_FOR_NETWORK_IDLE_TIMEOUT = 10000  # 10 秒

    def __init__(
        self,
        page: PlaywrightPage,
        wait_for_navigation_timeout: Optional[int] = None,
        wait_for_network_idle_timeout: Optional[int] = None,
    ):
        """
        初始化 Playwright 页面适配器

        Args:
            page: Playwright Page 实例
            wait_for_navigation_timeout: 导航超时时间（毫秒）
            wait_for_network_idle_timeout: 网络空闲超时时间（毫秒）
        """
        self.page = page
        self.wait_for_navigation_timeout = (
            wait_for_navigation_timeout or self.DEFAULT_WAIT_FOR_NAVIGATION_TIMEOUT
        )
        self.wait_for_network_idle_timeout = (
            wait_for_network_idle_timeout or self.DEFAULT_WAIT_FOR_NETWORK_IDLE_TIMEOUT
        )

        logger.debug(
            f"WebPage initialized: "
            f"nav_timeout={self.wait_for_navigation_timeout}ms, "
            f"idle_timeout={self.wait_for_network_idle_timeout}ms"
        )

    async def get_ui_context(self) -> UIContext:
        """获取当前 UI 上下文"""
        screenshot_data = await self.screenshot()
        size = await self.get_size()

        # 创建 UIContext（简化版，完整实现需要更多字段）
        class WebUIContext(UIContext):
            def __init__(self, screenshot: ScreenshotItem, size: Size):
                self.screenshot = screenshot
                self.size = size
                self._is_frozen = False

        return WebUIContext(
            screenshot=ScreenshotItem(screenshot_data),
            size=size
        )

    async def get_size(self) -> Size:
        """获取视口尺寸"""
        viewport = self.page.viewport_size
        if viewport is None:
            # 如果没有设置 viewport，尝试获取窗口尺寸
            size_info = await self.page.evaluate(
                "() => ({ width: window.innerWidth, height: window.innerHeight })"
            )
            return {
                "width": float(size_info["width"]),
                "height": float(size_info["height"]),
                "dpr": None
            }

        return {
            "width": float(viewport["width"]),
            "height": float(viewport["height"]),
            "dpr": None
        }

    async def screenshot(self, full_page: bool = False) -> str:
        """
        获取截图（Base64 编码）

        Args:
            full_page: 是否截取完整页面

        Returns:
            Base64 编码的图像字符串
        """
        logger.debug(f"Taking screenshot: full_page={full_page}")

        # 截图
        screenshot_bytes = await self.page.screenshot(
            type="png",
            full_page=full_page
        )

        # 转换为 Base64
        screenshot_base64 = base64.b64encode(screenshot_bytes).decode('utf-8')

        logger.debug(f"Screenshot taken: size={len(screenshot_base64)} chars")

        return screenshot_base64

    async def click(self, x: float, y: float) -> None:
        """
        点击指定坐标

        Args:
            x: X 坐标
            y: Y 坐标
        """
        logger.debug(f"Clicking at ({x}, {y})")

        await self.page.mouse.click(x, y)

        # 等待可能的导航
        await self.wait_for_navigation()

    async def input_text(
        self,
        text: str,
        x: Optional[float] = None,
        y: Optional[float] = None,
        clear_first: bool = True
    ) -> None:
        """
        输入文本

        Args:
            text: 要输入的文本
            x: 可选的 X 坐标（先点击再输入）
            y: 可选的 Y 坐标（先点击再输入）
            clear_first: 是否先清空输入框（默认 True，与 JS 版本一致）
        """
        logger.debug(f"Inputting text: '{text}' at ({x}, {y}), clear_first={clear_first}")

        # 如果提供了坐标，先点击
        if x is not None and y is not None:
            # 使用 mouse.click 而不是 self.click，避免等待导航
            await self.page.mouse.click(x, y)
            # 等待元素聚焦
            await asyncio.sleep(0.1)

        # 清空输入框内容（与 JS 版本一致）
        if clear_first:
            # 方法1: 全选后删除 (跨平台兼容)
            # 使用 Ctrl+A (Windows/Linux) 或 Meta+A (macOS) 全选
            try:
                # 尝试使用 Playwright 的 fill 方法定位到当前焦点元素
                # 先全选现有内容
                await self.page.keyboard.press("Control+a")
                await asyncio.sleep(0.05)
                # 删除选中内容
                await self.page.keyboard.press("Backspace")
                await asyncio.sleep(0.05)
            except Exception as e:
                logger.debug(f"Clear with Ctrl+A failed, trying alternative: {e}")
                # 备用方法：多次退格清除
                # 先移动到末尾，然后逐个删除
                await self.page.keyboard.press("End")
                for _ in range(100):  # 最多清除100个字符
                    await self.page.keyboard.press("Backspace")

        # 输入文本
        await self.page.keyboard.type(text)

    async def hover(self, x: float, y: float) -> None:
        """
        悬停到指定坐标

        Args:
            x: X 坐标
            y: Y 坐标
        """
        logger.debug(f"Hovering at ({x}, {y})")

        await self.page.mouse.move(x, y)

    async def scroll(
        self,
        direction: str,
        distance: Optional[int] = None
    ) -> None:
        """
        滚动页面

        Args:
            direction: 滚动方向（up/down/left/right）
            distance: 滚动距离（像素），默认为视口高度
        """
        logger.debug(f"Scrolling {direction}, distance={distance}")

        if distance is None:
            # 默认滚动一个视口的高度
            size = await self.get_size()
            distance = int(size["height"])

        # 根据方向计算滚动量
        delta_x = 0
        delta_y = 0

        if direction == "down":
            delta_y = distance
        elif direction == "up":
            delta_y = -distance
        elif direction == "right":
            delta_x = distance
        elif direction == "left":
            delta_x = -distance
        else:
            raise ValueError(f"Invalid scroll direction: {direction}")

        # 执行滚动
        await self.page.evaluate(
            f"window.scrollBy({delta_x}, {delta_y})"
        )

        # 等待滚动完成
        await asyncio.sleep(0.3)

    async def key_press(self, key: str) -> None:
        """
        按键

        Args:
            key: 按键名称（Enter、Escape 等）
        """
        logger.debug(f"Pressing key: {key}")

        await self.page.keyboard.press(key)

        # 等待可能的导航（如按 Enter 提交表单）
        await self.wait_for_navigation()

    async def wait_for_navigation(self, timeout: Optional[int] = None) -> None:
        """
        等待导航完成

        Args:
            timeout: 超时时间（毫秒）
        """
        timeout_ms = timeout or self.wait_for_navigation_timeout

        if timeout_ms == 0:
            logger.debug("Navigation timeout is 0, skipping wait")
            return

        logger.debug(f"Waiting for navigation (timeout={timeout_ms}ms)")

        try:
            # 等待 HTML 元素存在（表示页面已加载）
            await self.page.wait_for_selector(
                "html",
                timeout=timeout_ms
            )
        except Exception as e:
            logger.warning(
                f"Waiting for navigation timed out, but continuing execution: {e}"
            )

    async def wait_for_network_idle(self, timeout: Optional[int] = None) -> None:
        """
        等待网络空闲

        Args:
            timeout: 超时时间（毫秒）
        """
        timeout_ms = timeout or self.wait_for_network_idle_timeout

        if timeout_ms == 0:
            logger.debug("Network idle timeout is 0, skipping wait")
            return

        logger.debug(f"Waiting for network idle (timeout={timeout_ms}ms)")

        try:
            # Playwright 的网络空闲等待
            await self.page.wait_for_load_state(
                "networkidle",
                timeout=timeout_ms
            )
        except Exception as e:
            logger.warning(
                f"Waiting for network idle timed out, but continuing execution: {e}"
            )

    async def evaluate_javascript(self, script: str) -> Any:
        """
        执行 JavaScript 代码

        Args:
            script: JavaScript 代码

        Returns:
            执行结果
        """
        logger.debug(f"Evaluating JavaScript: {script[:100]}...")

        result = await self.page.evaluate(script)

        logger.debug(f"JavaScript evaluation result: {str(result)[:100]}...")

        return result

    async def wait(self, timeout_ms: int) -> None:
        """
        等待指定时间

        Args:
            timeout_ms: 等待时间（毫秒）
        """
        logger.debug(f"Waiting for {timeout_ms}ms")
        await asyncio.sleep(timeout_ms / 1000)

    # ==================== XPath 相关方法（与 JS 版本对齐） ====================

    async def get_element_xpath(self, x: float, y: float) -> Optional[str]:
        """
        获取指定坐标处元素的 XPath

        Args:
            x: X 坐标
            y: Y 坐标

        Returns:
            元素的 XPath 路径，如果找不到则返回 None
        """
        logger.debug(f"Getting XPath for element at ({x}, {y})")

        # JavaScript 代码：获取坐标处元素的 XPath
        xpath = await self.page.evaluate("""
            (coords) => {
                const { x, y } = coords;
                const element = document.elementFromPoint(x, y);
                if (!element) return null;

                // 生成 XPath 的函数
                function getXPath(el) {
                    if (!el || el.nodeType !== Node.ELEMENT_NODE) return '';
                    
                    // 如果有 ID，使用 ID 定位（更稳定）
                    // 注意：JS 版本不使用 ID，我们也保持一致
                    
                    const parts = [];
                    let current = el;
                    
                    while (current && current.nodeType === Node.ELEMENT_NODE) {
                        let index = 1;
                        let sibling = current.previousElementSibling;
                        
                        while (sibling) {
                            if (sibling.nodeName === current.nodeName) {
                                index++;
                            }
                            sibling = sibling.previousElementSibling;
                        }
                        
                        const tagName = current.nodeName.toLowerCase();
                        parts.unshift(`${tagName}[${index}]`);
                        current = current.parentElement;
                    }
                    
                    return '/' + parts.join('/');
                }

                return getXPath(element);
            }
        """, {"x": x, "y": y})

        if xpath:
            logger.debug(f"XPath found: {xpath}")
        else:
            logger.warning(f"No element found at ({x}, {y})")

        return xpath

    async def get_element_by_xpath(self, xpath: str) -> Optional[dict]:
        """
        通过 XPath 获取元素信息（边界框和中心点）

        Args:
            xpath: 元素的 XPath 路径

        Returns:
            包含 bbox 和 center 的字典，如果找不到则返回 None
        """
        logger.debug(f"Getting element by XPath: {xpath}")

        try:
            # 使用 Playwright 的 locator 通过 XPath 定位
            locator = self.page.locator(f"xpath={xpath}")
            
            # 检查元素是否存在
            count = await locator.count()
            if count == 0:
                logger.warning(f"Element not found for XPath: {xpath}")
                return None

            # 获取第一个匹配元素的边界框
            bounding_box = await locator.first.bounding_box()
            
            if bounding_box is None:
                logger.warning(f"Element has no bounding box: {xpath}")
                return None

            # 计算中心点
            center_x = bounding_box["x"] + bounding_box["width"] / 2
            center_y = bounding_box["y"] + bounding_box["height"] / 2

            result = {
                "bbox": [
                    bounding_box["x"],
                    bounding_box["y"],
                    bounding_box["x"] + bounding_box["width"],
                    bounding_box["y"] + bounding_box["height"]
                ],
                "center": [center_x, center_y],
                "rect": {
                    "left": bounding_box["x"],
                    "top": bounding_box["y"],
                    "width": bounding_box["width"],
                    "height": bounding_box["height"]
                }
            }

            logger.debug(f"Element found: center=({center_x}, {center_y})")
            return result

        except Exception as e:
            logger.warning(f"Failed to get element by XPath: {xpath}, error: {e}")
            return None

    async def click_by_xpath(self, xpath: str) -> bool:
        """
        通过 XPath 点击元素

        Args:
            xpath: 元素的 XPath 路径

        Returns:
            是否成功点击
        """
        logger.debug(f"Clicking element by XPath: {xpath}")

        try:
            locator = self.page.locator(f"xpath={xpath}")
            await locator.first.click()
            await self.wait_for_navigation()
            return True
        except Exception as e:
            logger.warning(f"Failed to click by XPath: {xpath}, error: {e}")
            return False

    async def input_by_xpath(self, xpath: str, text: str, clear_first: bool = True) -> bool:
        """
        通过 XPath 在元素中输入文本

        Args:
            xpath: 元素的 XPath 路径
            text: 要输入的文本
            clear_first: 是否先清空

        Returns:
            是否成功输入
        """
        logger.debug(f"Inputting text by XPath: {xpath}, text: '{text}'")

        try:
            locator = self.page.locator(f"xpath={xpath}")
            
            if clear_first:
                await locator.first.fill(text)
            else:
                await locator.first.type(text)
            
            return True
        except Exception as e:
            logger.warning(f"Failed to input by XPath: {xpath}, error: {e}")
            return False


__all__ = ["WebPage"]



"""
AI 模型服务调用器 - 对应 packages/core/src/ai-model/service-caller/index.ts

统一的 AI 模型调用入口，支持多种模型（OpenAI、千问等）。
"""

from typing import Optional, Dict, Any, List, Callable
import time
import json
from openai import OpenAI
from openai.types.chat import ChatCompletionMessageParam, ChatCompletion

from ...shared.logger import logger
from ...shared.utils import safe_parse_json, extract_json_from_code_block
from ..types import AIUsageInfo


# JSON 修复函数（使用 json-repair 库）
def repair_json(json_str: str) -> str:
    """
    修复不规范的 JSON 字符串

    Args:
        json_str: 需要修复的 JSON 字符串

    Returns:
        修复后的 JSON 字符串
    """
    try:
        from json_repair import repair_json as _repair
        return _repair(json_str)
    except ImportError:
        logger.warning("json-repair not installed, skipping JSON repair")
        return json_str


def safe_parse_json_with_repair(text: str) -> Optional[Dict[str, Any]]:
    """
    安全解析 JSON，支持自动修复

    Args:
        text: JSON 字符串

    Returns:
        解析后的字典，失败返回 None
    """
    # 首先尝试直接解析
    result = safe_parse_json(text)
    if result is not None:
        return result

    # 尝试提取代码块
    extracted = extract_json_from_code_block(text)
    result = safe_parse_json(extracted)
    if result is not None:
        return result

    # 尝试修复后解析
    try:
        repaired = repair_json(extracted)
        return json.loads(repaired)
    except Exception as e:
        logger.error(f"Failed to parse JSON even after repair: {e}")
        return None


class ModelConfig:
    """模型配置"""

    def __init__(
        self,
        model_name: str = "qwen-vl-max",
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
        temperature: float = 0.0,
        max_tokens: int = 4096,
        timeout: int = 60000,  # 毫秒
        retry_count: int = 1,
        retry_interval: int = 2000,  # 毫秒
        http_proxy: Optional[str] = None,
        model_description: Optional[str] = None,
        intent: Optional[str] = None,
        extra_config: Optional[Dict[str, Any]] = None,
    ):
        self.model_name = model_name
        self.base_url = base_url
        self.api_key = api_key
        self.temperature = temperature
        self.max_tokens = max_tokens
        self.timeout = timeout
        self.retry_count = retry_count
        self.retry_interval = retry_interval
        self.http_proxy = http_proxy
        self.model_description = model_description or model_name
        self.intent = intent
        self.extra_config = extra_config or {}


def create_chat_client(model_config: ModelConfig) -> OpenAI:
    """
    创建 OpenAI 客户端

    Args:
        model_config: 模型配置

    Returns:
        OpenAI 客户端实例
    """
    client_kwargs = {
        "api_key": model_config.api_key,
        "timeout": model_config.timeout / 1000,  # 转换为秒
    }

    # 设置 base_url
    if model_config.base_url:
        client_kwargs["base_url"] = model_config.base_url

    # 设置代理
    if model_config.http_proxy:
        import httpx
        client_kwargs["http_client"] = httpx.Client(
            proxy=model_config.http_proxy
        )
        logger.debug(f"Using HTTP proxy: {model_config.http_proxy}")

    # 应用额外配置
    client_kwargs.update(model_config.extra_config)

    return OpenAI(**client_kwargs)


def build_usage_info(
    usage_data: Any,
    time_cost: float,
    model_config: ModelConfig
) -> AIUsageInfo:
    """
    构建 AI 使用信息

    Args:
        usage_data: OpenAI 返回的 usage 数据
        time_cost: 耗时（毫秒）
        model_config: 模型配置

    Returns:
        AIUsageInfo 实例
    """
    return AIUsageInfo(
        prompt_tokens=getattr(usage_data, "prompt_tokens", None),
        completion_tokens=getattr(usage_data, "completion_tokens", None),
        total_tokens=getattr(usage_data, "total_tokens", None),
        cached_input=getattr(
            getattr(usage_data, "prompt_tokens_details", None),
            "cached_tokens",
            None
        ),
        time_cost=time_cost,
        model_name=model_config.model_name,
        model_description=model_config.model_description,
        intent=model_config.intent,
    )


def call_ai(
    messages: List[ChatCompletionMessageParam],
    model_config: ModelConfig,
    stream: bool = False,
    on_chunk: Optional[Callable[[str], None]] = None,
) -> Dict[str, Any]:
    """
    调用 AI 模型

    Args:
        messages: 消息列表
        model_config: 模型配置
        stream: 是否使用流式输出
        on_chunk: 流式输出回调函数

    Returns:
        包含 content, usage, isStreamed 的字典
    """
    client = create_chat_client(model_config)

    start_time = time.time()
    max_attempts = model_config.retry_count + 1
    last_error: Optional[Exception] = None

    # 准备请求参数
    request_params = {
        "model": model_config.model_name,
        "messages": messages,
        "temperature": model_config.temperature,
        "max_tokens": model_config.max_tokens,
    }

    # 千问 VL 特定配置
    if "qwen" in model_config.model_name.lower() and "vl" in model_config.model_name.lower():
        request_params["extra_body"] = {
            "vl_high_resolution_images": True
        }

    logger.info(f"Sending request to {model_config.model_name}")

    # 重试逻辑
    for attempt in range(1, max_attempts + 1):
        try:
            if stream and on_chunk:
                # 流式调用
                accumulated = ""
                usage = None

                response_stream = client.chat.completions.create(
                    **request_params,
                    stream=True
                )

                for chunk in response_stream:
                    if chunk.choices and len(chunk.choices) > 0:
                        delta = chunk.choices[0].delta
                        if delta.content:
                            accumulated += delta.content
                            on_chunk(delta.content)

                        # 检查是否有 usage 信息
                        if hasattr(chunk, 'usage') and chunk.usage:
                            usage = chunk.usage

                time_cost = (time.time() - start_time) * 1000

                return {
                    "content": accumulated,
                    "usage": build_usage_info(usage, time_cost, model_config) if usage else None,
                    "isStreamed": True,
                }

            else:
                # 非流式调用
                response: ChatCompletion = client.chat.completions.create(
                    **request_params
                )

                time_cost = (time.time() - start_time) * 1000

                # 提取响应内容
                if not response.choices:
                    raise ValueError(
                        f"Invalid response from LLM service: {response}"
                    )

                content = response.choices[0].message.content or ""

                # 构建使用信息
                usage_info = None
                if response.usage:
                    usage_info = build_usage_info(
                        response.usage,
                        time_cost,
                        model_config
                    )

                logger.info(
                    f"Model response received: "
                    f"model={model_config.model_name}, "
                    f"tokens={response.usage.total_tokens if response.usage else 0}, "
                    f"cost_ms={time_cost:.0f}"
                )

                return {
                    "content": content,
                    "usage": usage_info,
                    "isStreamed": False,
                }

        except Exception as e:
            last_error = e
            logger.error(
                f"Attempt {attempt}/{max_attempts} failed: {str(e)}"
            )

            if attempt < max_attempts:
                # 等待后重试
                retry_delay = model_config.retry_interval / 1000
                logger.info(f"Retrying in {retry_delay} seconds...")
                time.sleep(retry_delay)
            else:
                # 所有重试都失败
                logger.error(f"All {max_attempts} attempts failed")
                raise last_error


def extract_json_from_response(content: str) -> Optional[Dict[str, Any]]:
    """
    从 AI 响应中提取 JSON

    Args:
        content: AI 响应内容

    Returns:
        解析后的 JSON 对象，失败返回 None
    """
    return safe_parse_json_with_repair(content)


__all__ = [
    "ModelConfig",
    "create_chat_client",
    "call_ai",
    "extract_json_from_response",
    "safe_parse_json_with_repair",
]
